@@ -18,8 +18,8 @@
 #ifndef _UDM_CHINESE_H
 #define _UDM_CHINESE_H
 
-extern char *UdmSegmentByFreq(UDM_CHINALIST *List, char *line);
+extern int *UdmSegmentByFreq(UDM_CHINALIST *List, int *line);
 extern __C_LINK void __UDMCALL UdmChineseListFree(UDM_CHINALIST *List);
-extern int UdmChineseListLoad(UDM_CHINALIST *List, const char * chineselist_filename);
+extern int UdmChineseListLoad(UDM_AGENT *Agent, UDM_CHINALIST *List, const char *charset, const char * chineselist_filename);
 
 #endif
