@@ -83,7 +83,6 @@
 /* storage types */
 #define UDM_DBMODE_SINGLE	0
 #define UDM_DBMODE_MULTI	1
-#define UDM_DBMODE_FLY          2
 #define UDM_DBMODE_BLOB		6
 
 
@@ -108,6 +107,8 @@
 #define UDM_MATCH_REGEX		4
 #define UDM_MATCH_WILD		5
 #define UDM_MATCH_SUBNET	6
+#define UDM_MATCH_NUMERIC_LT	7
+#define UDM_MATCH_NUMERIC_GT	8
 
 /* Indexer return codes */
 #define UDM_OK			0
@@ -149,6 +150,7 @@
 #define UDM_METHOD_VISITLATER	7
 #define UDM_METHOD_INDEX	8
 #define UDM_METHOD_NOINDEX	9
+#define UDM_METHOD_IMPORTONLY	10
 
 /* Tags/categories defines */
 #define UDM_CATSIZE	8
@@ -223,12 +225,13 @@ typedef struct udm_varlist_st {
 } UDM_VARLIST;
 
 
-
+#define UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION 1
 typedef struct {
 	char		*str;
 	char		*href;
 	char		*section_name;
 	int		section;
+	int		flags;
 } UDM_TEXTITEM;
 
 typedef struct {
@@ -266,6 +269,8 @@ typedef struct{
 
 typedef struct {
         urlid_t         url_id;
+        uint4           coord;
+        uint4           per_site;
         urlid_t         site_id;
         time_t          last_mod_time;
         double          pop_rank;
@@ -279,6 +284,7 @@ typedef struct {
 } UDM_URLDATALIST;
 
 typedef struct {
+        size_t          acoords;
 	size_t		ncoords;
 	size_t		order;
 	char		*word;
@@ -409,6 +415,7 @@ typedef struct {
         urlid_t server_id;
         urlid_t rec_id;
         size_t  max_doc_per_site;
+        UDM_VARLIST Vars;
 } UDM_HREF;
 
 typedef struct {
@@ -513,6 +520,7 @@ typedef struct udm_parser_struct{
         char		*from_mime;
 	char		*to_mime;
 	char		*cmd;
+	char		*src;
 } UDM_PARSER;
 
 typedef struct {
@@ -534,12 +542,17 @@ typedef struct udm_spell_st
 #define UDM_SPELL_LANGLEN 32
 #define UDM_SPELL_CSETLEN 32
 #define UDM_SPELL_FILELEN 128
+#define UDM_SPELL_FMT_TEXT 0
+#define UDM_SPELL_FMT_HASH 1
 
 typedef struct udm_dict_st
 {
   char  lang[UDM_SPELL_LANGLEN];
   char  cset[UDM_SPELL_CSETLEN];
   char  fname[UDM_SPELL_FILELEN];
+  int   fmt;
+  int   fd;
+  size_t wordlen;
   UDM_CHARSET *cs;
   char   *fbody;
   size_t nitems;
@@ -629,6 +642,8 @@ typedef struct {
 	size_t		len;
 	size_t		ulen;
         int     	origin;
+        int             weight; /* origin-dependent weight */
+        int		match;
 } UDM_WIDEWORD;
 
 typedef struct {
@@ -709,7 +724,6 @@ typedef struct {
 	size_t			cur_row;
 /*	size_t			offset;*/
 	size_t			memused;
-        size_t                  *PerSite;
 	int			freeme;
 	UDM_DOCUMENT		*Doc;
 	
@@ -751,13 +765,19 @@ typedef struct udm_config_struct {
 	UDM_MATCHLIST	MimeTypes;	/**< For AddType commands       */
 	UDM_MATCHLIST	Filters;	/**< Allow, Disallow,etc        */
 	UDM_MATCHLIST	SectionFilters; /**< IndexIf, NoIndexIf         */
-        UDM_MATCHLIST   SectionMatch;   /**< User defined sections      */
+        UDM_MATCHLIST   SectionHdrMatch;/**< User sections after headers*/
+        UDM_MATCHLIST   SectionGsrMatch;/**< User sections after quesser*/
+        UDM_MATCHLIST   SectionMatch;   /**< User sections after parser */
 	
 	UDM_HREFLIST	Hrefs;		/**< Links cache                */
 	UDM_RESULT	Targets;	/**< Targets cache              */
 	
 	UDM_VARLIST	Sections;	/**< document section to parse  */
 	UDM_VARLIST	Vars;		/**< Config parameters          */
+	UDM_VARLIST	Cookies;	/**< Cookie list                */
+	UDM_VARLIST	XMLEnterHooks;
+	UDM_VARLIST	XMLLeaveHooks;
+	UDM_VARLIST	XMLDataHooks;
 	
 	UDM_LANGMAPLIST	LangMaps;	/**< For lang+charset quesser   */
 	UDM_ROBOTS	Robots;		/**< robots.txt information     */
@@ -831,7 +851,7 @@ typedef struct {
 	int	status;
 } UDM_URLSTATE;
 
-typedef int (*qsort_cmp)(const void*, const void*);
+typedef int (*udm_qsort_cmp)(const void*, const void*);
 
 typedef struct {
 	uint4	hi,lo;
@@ -956,7 +976,11 @@ enum udm_indcmd
   UDM_IND_CONVERT,
   UDM_IND_MULTI2BLOB,
   UDM_IND_EXPORT,
-  UDM_IND_WRDSTAT
+  UDM_IND_WRDSTAT,
+  UDM_IND_REWRITEURL,
+  UDM_IND_HASHSPELL,
+  UDM_IND_DUMPSPELL,
+  UDM_IND_REWRITELIMITS
 };
 
 #endif
