@@ -35,28 +35,29 @@
 #define UDM_DB_ACCESS			14
 #define UDM_DB_MIMER			15
 #define UDM_DB_CACHE			16
+#define UDM_DB_SYBASE			17
 #define UDM_DB_SEARCHD			200
 
-#define UDM_URL_ACTION_DELETE		1
-#define UDM_URL_ACTION_ADD		2
-#define UDM_URL_ACTION_SUPDATE		3
-#define UDM_URL_ACTION_LUPDATE		4
-#define UDM_URL_ACTION_DUPDATE		5
-#define UDM_URL_ACTION_UPDCLONE		9
-#define UDM_URL_ACTION_REGCHILD		10
-#define UDM_URL_ACTION_FINDBYURL	11
-#define UDM_URL_ACTION_FINDBYMSG	12
-#define UDM_URL_ACTION_FINDORIG		13
-#define UDM_URL_ACTION_EXPIRE		14
-#define UDM_URL_ACTION_REFERERS		15
-#define UDM_URL_ACTION_SERVERTABLE	17
-#define UDM_URL_ACTION_DOCCOUNT		18
-#define UDM_URL_ACTION_FLUSH		19
-#define UDM_URL_ACTION_LINKS_DELETE     21
-#define UDM_URL_ACTION_ADD_LINK		22
-#define UDM_URL_ACTION_GET_CACHED_COPY	23
-#define UDM_URL_ACTION_WRDSTAT		24
-#define UDM_URL_ACTION_DOCPERSITE	25
+#define UDM_URL_ACTION_DELETE           0
+#define UDM_URL_ACTION_ADD              1
+#define UDM_URL_ACTION_SUPDATE          2
+#define UDM_URL_ACTION_LUPDATE          3
+#define UDM_URL_ACTION_DUPDATE          4
+#define UDM_URL_ACTION_UPDCLONE         5
+#define UDM_URL_ACTION_REGCHILD         6
+#define UDM_URL_ACTION_FINDBYURL        7
+#define UDM_URL_ACTION_FINDBYMSG        8
+#define UDM_URL_ACTION_FINDORIG         9
+#define UDM_URL_ACTION_EXPIRE           10
+#define UDM_URL_ACTION_REFERERS         11
+#define UDM_URL_ACTION_DOCCOUNT         12
+#define UDM_URL_ACTION_LINKS_DELETE     13
+#define UDM_URL_ACTION_ADD_LINK         14
+#define UDM_URL_ACTION_GET_CACHED_COPY  15
+#define UDM_URL_ACTION_WRDSTAT          16
+#define UDM_URL_ACTION_DOCPERSITE       17
+#define UDM_URL_ACTION_SQLIMPORTSEC     18
+#define UDM_URL_ACTION_FLUSH            19
 
 #define UDM_RES_ACTION_DOCINFO		1
 #define UDM_RES_ACTION_INSWORDS		2
@@ -126,7 +127,6 @@ extern __C_LINK UDM_DBLIST	* __UDMCALL UdmDBListInit(UDM_DBLIST *List);
 extern __C_LINK size_t	__UDMCALL UdmDBListAdd(UDM_DBLIST *List, const char * addr, int mode);
 
 extern __C_LINK void * __UDMCALL UdmDBInit(void *db);
-extern __C_LINK int __UDMCALL UdmGetDocCount(UDM_AGENT * Indexer,UDM_DB *db);
 
 /******************/
 
@@ -134,8 +134,10 @@ extern __C_LINK int __UDMCALL UdmGetDocCount(UDM_AGENT * Indexer,UDM_DB *db);
 /******************/
 
 #ifdef HAVE_SQL
+extern int (*udm_url_action_handlers[])(UDM_AGENT *A, UDM_DOCUMENT *D, UDM_DB *db);
+#define UdmURLActionSQL(A, D, cmd, db) udm_url_action_handlers[cmd](A, D, db)
+
 extern int   UdmClearDBSQL(UDM_AGENT    *A, UDM_DB *db);
-extern int   UdmURLActionSQL(UDM_AGENT  *A, UDM_DOCUMENT   *D, int cmd, UDM_DB *db);
 extern int   UdmResActionSQL(UDM_AGENT  *A, UDM_RESULT     *R, int cmd, UDM_DB *db, size_t dbnum);
 extern int   UdmCatActionSQL(UDM_AGENT  *A, UDM_CATEGORY   *C, int cmd, UDM_DB *db);
 extern int   UdmSrvActionSQL(UDM_AGENT  *A, UDM_SERVERLIST *S, int cmd, UDM_DB *db);
@@ -148,7 +150,6 @@ extern unsigned int   UdmGetCategoryIdSQL(UDM_ENV *Conf, char *category, UDM_DB
 extern int UdmResAddDocInfoSQL(UDM_AGENT *query, UDM_DB *db, UDM_RESULT *Res, size_t i);
 extern int UdmTrackSQL(UDM_AGENT * query, UDM_RESULT *Res, UDM_DB *db);
 extern int UdmCloneListSQL(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc, UDM_RESULT *Res, UDM_DB *db);
-extern int UdmLoadURLDataSQL(UDM_AGENT *A, UDM_RESULT *R, UDM_DB *db);
 extern int UdmTargetsSQL(UDM_AGENT *Indexer, UDM_DB *db);
 extern int UdmUserCacheStoreSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
 #endif
@@ -160,7 +161,14 @@ extern size_t UdmMultiCacheAdd (UDM_MULTI_CACHE *cache, urlid_t url_id, unsigned
 
 extern int UdmMulti2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
 extern int UdmSingle2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
+extern int UdmBlob2BlobSQL (UDM_AGENT *Indexer, UDM_DB *db);
 extern __C_LINK int __UDMCALL UdmMulti2Blob (UDM_AGENT *Indexer);
+extern __C_LINK int __UDMCALL UdmRewriteURL(UDM_AGENT *Indexer);
+extern __C_LINK int __UDMCALL UdmRewriteLimits(UDM_AGENT *Indexer);
+extern int UdmBlobWriteURL(UDM_AGENT *A, UDM_DB *db, const char *table,
+    int use_deflate);
+extern int UdmBlobWriteLimits(UDM_AGENT *A, UDM_DB *db, const char *table,
+    int use_deflate);
 extern UDM_BLOB_CACHE *UdmBlobCacheInit (UDM_BLOB_CACHE *cache);
 extern void UdmBlobCacheFree (UDM_BLOB_CACHE *cache);
 extern size_t UdmBlobCacheAdd (UDM_BLOB_CACHE *cache, urlid_t url_id,
@@ -177,10 +185,6 @@ extern int UdmWordCacheAddURL (UDM_WORD_CACHE *cache, urlid_t url_id);
 extern int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit);
 extern int UdmWordCacheFlush (UDM_AGENT *Indexer);
 
-/* DBModeFly */
-extern int UdmDBModeFlyWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit);
-extern int UdmDBModeFlyMerge (UDM_AGENT *Indexer, UDM_DB *db);
-
 extern int UdmExport (UDM_AGENT *Indexer);
 extern int UdmExportSQL (UDM_AGENT *Indexer, UDM_DB *db);
 
