@@ -23,6 +23,8 @@ extern __C_LINK void __UDMCALL UdmDocFree(UDM_DOCUMENT*);
 extern int           UdmDocToTextBuf(UDM_DOCUMENT * Doc,char *d,size_t maxlen);
 extern __C_LINK int __UDMCALL UdmDocFromTextBuf(UDM_DOCUMENT * Doc,const char *d);
 
+extern void UdmURLCRDListListInit(UDM_URLCRDLISTLIST *Lst);
 extern void UdmURLCRDListListFree(UDM_URLCRDLISTLIST *Lst);
+extern int UdmURLCRDListListAdd(UDM_URLCRDLISTLIST *Lst, UDM_URLCRDLIST *I);
 
 #endif
