@@ -22,7 +22,7 @@
 #define FTP_BUSY	1
 
 extern int Udm_ftp_connect(UDM_ENV * Conf,UDM_CONN *connp, 
-	char *hostname, int port, char *user, char *passwd, int timeout);
+	const char *hostname, int port, char *user, char *passwd, int timeout);
 extern int Udm_ftp_open_control_port(UDM_ENV * Conf,UDM_CONN *connp);
 
 extern int Udm_ftp_get_reply(UDM_CONN *connp);
@@ -31,7 +31,7 @@ extern int Udm_ftp_open_data_port(UDM_CONN *c, UDM_CONN *d);
 extern int Udm_ftp_send_cmd(UDM_CONN *connp, const char *cmd);
 extern int Udm_ftp_send_data_cmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size);
 extern int Udm_ftp_login(UDM_CONN *connp, char *user, char *passwd);
-extern int Udm_ftp_cwd(UDM_CONN *c, char *path);
+extern int Udm_ftp_cwd(UDM_CONN *c, const char *path);
 extern int Udm_ftp_list(UDM_DOCUMENT *Doc);
 extern int Udm_ftp_get(UDM_CONN *c, UDM_CONN *d, char *path,size_t max_doc_size);
 extern int Udm_ftp_mdtm(UDM_CONN *c, char *path);
