@@ -42,6 +42,6 @@ extern  int  UdmGuessCharSet(UDM_AGENT *Indexer, UDM_DOCUMENT * D, UDM_LANGMAPLI
 extern int   UdmLoadLangMapList(UDM_LANGMAPLIST *L, const char * mapdir);
 extern __C_LINK int __UDMCALL UdmLoadLangMapFile(UDM_LANGMAPLIST *L, const char * mapname);
 
-extern const char *UdmGuessContentType(const char *beg, size_t len);
+extern const char *UdmGuessContentType(const char *beg, size_t len, const char *def);
 
 #endif
