@@ -22,7 +22,8 @@
 
 extern __C_LINK int  __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer);
 extern __C_LINK int  __UDMCALL UdmURLFile(UDM_AGENT *Indexer,const char *f, int cmd);
-extern int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL, UDM_HREF *Href);
+extern int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL,
+                          UDM_SPIDERPARAM *Spider, UDM_HREF *Href);
 extern __C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT *Indexer);
 extern int UdmDocStoreHrefs(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc);
 
