@@ -33,6 +33,9 @@ extern int  UdmPrepare(UDM_AGENT *query,UDM_RESULT *res);
 extern __C_LINK int __UDMCALL UdmParseQueryString(UDM_AGENT * Agent,UDM_VARLIST * vars,char * query_string);
 
 extern char *UdmHlConvert(UDM_WIDEWORDLIST *L,const char * src, UDM_CHARSET * lcs, UDM_CHARSET * bcs);
+extern char *UdmHlConvertExt(UDM_WIDEWORDLIST *L, const char * src,
+                             UDM_CHARSET * lcs, UDM_CHARSET * bcs,
+                             int hilight_stopwords);
 extern int  UdmConvert(UDM_ENV *Conf, UDM_RESULT *Res,UDM_CHARSET *lcs,UDM_CHARSET *bcs);
 extern char* UdmRemoveHiLightDup(const char *s);
 
@@ -41,4 +44,6 @@ extern int UdmCatFromTextBuf(UDM_CATEGORY *C, char *textbuf);
 
 int   *UdmUniSegment(UDM_AGENT *Indexer, int *s, const char *lang);
 
+int UdmResWordInfo(UDM_ENV *Env, UDM_RESULT *Res);
+
 #endif
