@@ -18,12 +18,15 @@
 #ifndef _UDM_SIGNALS_H
 #define _UDM_SIGNALS_H
 
+#include "udm_common.h"
+
 /*
  * udm_signals.h from UdmSearch
  * (C) 2000 Kir <kir@sever.net>, UdmSearch Developers Team
  */
 
 extern int UdmSigHandlersInit(UDM_AGENT *Indexer);
+extern void (*UdmSignal(int signo, void (*handler)(int)))(int);
 
 extern int have_sighup;
 extern int have_sigterm;
