@@ -18,8 +18,14 @@
 #ifndef _UDM_TEMPLATE_H
 #define _UDM_TEMPLATE_H
 
-extern int  UdmTemplateLoad(UDM_ENV * Env, UDM_VARLIST *vars, const char *tname, UDM_VARLIST *tmpl);
+extern int  UdmTemplateLoad(UDM_AGENT *Agent, const char *tname, UDM_VARLIST *tmpl);
 extern __C_LINK void __UDMCALL UdmTemplatePrint(UDM_AGENT * Agent, FILE *stream, char *dst, size_t dst_len, 
 			     UDM_VARLIST *vars, UDM_VARLIST *t, const char *where);
-
+extern __C_LINK size_t __UDMCALL UdmTemplatePrintVar(UDM_ENV *Conf,
+                                                     FILE *stream,
+                                                     char *dst, size_t dlen,
+                                                     const char *val,
+                                                     int type,
+                                                     const char *hlbeg,
+                                                     const char *hlend);
 #endif
