@@ -69,7 +69,6 @@ This is C version of a LDC Chinese segmentor
 #include "udm_chinese.h"
 #include "udm_utils.h"
 #include "udm_xmalloc.h"
-#include "udm_log.h"
 #include "udm_unicode.h"
 #include "udm_unidata.h"
 
@@ -143,18 +142,20 @@ int UdmChineseListLoad(UDM_AGENT *Agent, UDM_CHINALIST *List, const char *charse
      int uword[256];
      UDM_CHARSET *sys_int, *fcs;
      UDM_CONV to_uni;
-
-     sys_int = UdmGetCharSet("sys-int");
+     UDM_ENV *Env= Agent->Conf;
+     
+     sys_int = &udm_charset_sys_int;
      if (!(fcs = UdmGetCharSet(charset))) {
-       UdmLog(Agent, UDM_LOG_ERROR, "Charset '%s' not found or not supported", charset);
+       udm_snprintf(Env->errstr, sizeof(Env->errstr),
+                    "Charset '%s' not found or not supported", charset);
        return UDM_ERROR;
      }
      UdmConvInit(&to_uni, fcs, sys_int, UDM_RECODE_HTML);
 
-
-
      if (!(file = fopen(fname, "r"))) {
-          UdmLog(Agent, UDM_LOG_ERROR, "Can't open frequency dictionary file '%s' (%s)", fname, strerror(errno));
+          udm_snprintf(Env->errstr, sizeof(Env->errstr),
+                       "Can't open frequency dictionary file '%s' (%s)",
+                       fname, strerror(errno));
           return UDM_ERROR;
      }
      bzero((void*)&chinaword, sizeof(chinaword));
