@@ -114,6 +114,7 @@ const char *UdmMethodStr(int method)
     case UDM_METHOD_VISITLATER:  return "Skip";
     case UDM_METHOD_INDEX:    return "IndexIf";
     case UDM_METHOD_NOINDEX:  return "NoIndexIf";
+    case UDM_METHOD_IMPORTONLY:  return "ImportOnly";
   }
   return "<Unknown method>";
 }
@@ -128,8 +129,10 @@ int UdmMethod(const char *s)
   if(!strcasecmp(s,"CheckOnly"))    return UDM_METHOD_HEAD;
   if(!strcasecmp(s,"HrefOnly"))    return UDM_METHOD_HREFONLY;
   if(!strcasecmp(s,"Skip"))    return UDM_METHOD_VISITLATER;
+  if(!strcasecmp(s,"SkipIf"))    return UDM_METHOD_VISITLATER;
   if(!strcasecmp(s,"IndexIf"))    return UDM_METHOD_INDEX;
   if(!strcasecmp(s,"NoIndexIf"))    return UDM_METHOD_NOINDEX;
+  if(!strcasecmp(s,"ImportOnly"))   return UDM_METHOD_IMPORTONLY;
   return UDM_METHOD_UNKNOWN;
 }
 
@@ -290,7 +293,10 @@ static int add_srv(void *Cfg, size_t ac,char **av)
     if(!strcasecmp(av[i],"string"))C->Srv->Match.match_type=UDM_MATCH_WILD;
     else
     if(!strcasecmp(av[i],"regex"))C->Srv->Match.match_type=UDM_MATCH_REGEX;
-    else{
+    else
+    if(!strcasecmp(av[i],"regexp"))C->Srv->Match.match_type=UDM_MATCH_REGEX;
+    else
+    {
       if(!C->Srv->Match.pattern)
         C->Srv->Match.pattern = (char*)UdmStrdup(av[i]);
       else
@@ -323,7 +329,7 @@ static int add_srv(void *Cfg, size_t ac,char **av)
   {
     UDM_HREF Href;
     
-    bzero((void*)&Href, sizeof(Href));
+    UdmHrefInit(&Href);
     Href.url=C->Srv->Match.pattern;
     Href.method=UDM_METHOD_GET;
     Href.site_id = C->Srv->site_id;
@@ -357,7 +363,7 @@ static int UdmMatchListAddWithServer(UDM_AGENT *A,
     n.Match.match_type= M->match_type;
     n.Match.case_sense= M->case_sense;
     n.Match.nomatch= M->nomatch;
-    n.Match.arg= M->arg ? (char*)UdmStrdup(M->arg) : NULL;
+    n.Match.arg= M->arg;
     n.ordre= ordre;
     
     rc= UdmSrvAction(A, &S, UDM_SRV_ACTION_ADD);
@@ -585,6 +591,7 @@ static int add_parser(void *Cfg, size_t ac,char **av)
   P.from_mime=av[1];
   P.to_mime=av[2];
   P.cmd = av[3] ? av[3] : empty;
+  P.src= av[4];
   UdmParserAdd(&Conf->Parsers,&P);
   return UDM_OK;
 }
@@ -608,6 +615,7 @@ static int add_section(void *Cfg, size_t ac,char **av)
   UDM_ENV  *Conf=C->Indexer->Conf;
   UDM_VAR  S;
   int      cdon;
+  UDM_MATCHLIST *SectionMatch= &Conf->SectionMatch;
 
   bzero((void*)&S, sizeof(S));
   S.name= av[1];
@@ -628,6 +636,12 @@ static int add_section(void *Cfg, size_t ac,char **av)
       cdon= 1;
     else if (!strcasecmp(av[0], "cdoff") || !strcasecmp(av[0], "NoDetectClones"))
       cdon= 0;
+    else if (!strcasecmp(av[0], "afterheaders"))
+      SectionMatch= &Conf->SectionHdrMatch;
+    else if (!strcasecmp(av[0], "afterguesser"))
+      SectionMatch= &Conf->SectionGsrMatch;
+    else if (!strcasecmp(av[0], "afterparser"))
+      SectionMatch= &Conf->SectionMatch;
     else
       break;
   }
@@ -679,7 +693,7 @@ static int add_section(void *Cfg, size_t ac,char **av)
         break;
     }
 
-    if(UDM_OK != UdmMatchListAdd(C->Indexer, &Conf->SectionMatch, &M,
+    if(UDM_OK != UdmMatchListAdd(C->Indexer, SectionMatch, &M,
                                  err, sizeof(err), ++C->ordre))
     {
       udm_snprintf(Conf->errstr,sizeof(Conf->errstr)-1,"%s",err);
@@ -805,7 +819,13 @@ static int add_chinese(void *Cfg, size_t ac,char **av)
 {
   UDM_CFG  *C=(UDM_CFG*)Cfg;
   UDM_ENV  *Conf=C->Indexer->Conf;
-  if(C->flags & UDM_FLAG_ADD_SERV)
+  
+  /*
+    This line was wrong: ChinesList was not really loaded
+    from search.cgi
+  */
+  /* if(C->flags & UDM_FLAG_ADD_SERV)*/
+  
   {
     char fname[1024];
     rel_etc_name(Conf, fname, sizeof(fname)-1, av[2] ? av[2] : "mandarin.freq");
@@ -882,14 +902,23 @@ static int add_limit(void *Cfg, size_t ac,char **av)
   UDM_ENV  *Conf=C->Indexer->Conf;
   char * sc;
   char * nm;
-  
-  if((sc = strchr(av[1],':')))
+
+  if (ac == 2)
   {
-    *sc++='\0';
-    nm=(char*)UdmMalloc(strlen(av[1])+8);
-    sprintf(nm,"Limit-%s",av[1]);
-    UdmVarListReplaceStr(&Conf->Vars, nm, sc);
-    UDM_FREE(nm);
+    if((sc = strchr(av[1],':')))
+    {
+      *sc++='\0';
+      nm=(char*)UdmMalloc(strlen(av[1])+8);
+      sprintf(nm,"Limit-%s",av[1]);
+      UdmVarListReplaceStr(&Conf->Vars, nm, sc);
+      UDM_FREE(nm);
+    }
+  }
+  else if (ac == 3)
+  {
+    char name[128];
+    udm_snprintf(name, sizeof(name), "Limit.%s", av[1]);
+    UdmVarListReplaceStr(&Conf->Vars, name, av[2]);
   }
   return UDM_OK;
 }
@@ -992,39 +1021,30 @@ static int srv_rpl_auth(void *Cfg, size_t ac,char **av)
 
 char *UdmParseEnvVar (UDM_ENV *Conf, const char *str)
 {
-  char *p1 = (char *)str;
-  char *p2 = (char *)str;
-  char *p3;
-  const char *s;
-  char *o = NULL;
-
-  if (! str) return(NULL);
-  while ((p1 = strchr(p1, '$')))
+  const char *p1= str, *p2= str;
+  UDM_DSTR rc;
+  UdmDSTRInit(&rc, 256);
+  while ((p1= strstr(p1, "$(")))
   {
-    if (p1[1] != '(')
+    char *p3;
+    UdmDSTRAppend(&rc, p2, p1 - p2);
+    if ((p3= strchr(p1 + 2, ')')))
     {
-      p1++;
-      continue;
-    }
-    *p1 = 0;
-    o = UdmStrStore(o, p2);
-    *p1 = '$';
-    if ((p3 = strchr(p1 + 2, ')')))
-    {
-      *p3 = 0;
-      s = UdmVarListFindStr(&Conf->Vars, p1 + 2, NULL);
-      if (s) o = UdmStrStore(o, s);
-      *p3 = ')';
-      p2 = p1 = p3 + 1;
+      const char *s;
+      *p3= 0;
+      if ((s= UdmVarListFindStr(&Conf->Vars, p1 + 2, NULL)))
+        UdmDSTRAppendSTR(&rc, s);
+      *p3= ')';
+      p1= p2= p3 + 1;
     }
     else
     {
-      UDM_FREE(o);
+      UdmDSTRFree(&rc);
       return(NULL);
     }
   }
-  o = UdmStrStore(o, p2);
-  return(o);
+  UdmDSTRAppendSTR(&rc, p2);
+  return(rc.data);
 }
 
 static int env_rpl_env_var (void *Cfg, size_t ac, char **av)
@@ -1076,6 +1096,28 @@ static int env_rpl_var(void *Cfg, size_t ac,char **av)
   return UDM_OK;
 }
 
+
+static int env_rpl_named_var(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG  *C=(UDM_CFG*)Cfg;
+  UDM_ENV  *Conf=C->Indexer->Conf;
+  UdmVarListReplaceStr(&Conf->Vars,av[1],av[2]);
+  return UDM_OK;
+}
+
+
+static int rpl_xml_hook(void *Cfg, size_t ac,char **av)
+{
+  UDM_CFG  *C=(UDM_CFG*)Cfg;
+  UDM_ENV  *Conf=C->Indexer->Conf;
+  UDM_VARLIST *Vars= !strcasecmp(av[0], "XMLEnterHook") ?
+                     &Conf->XMLEnterHooks : 
+                     !strcasecmp(av[0], "XMLDataHook") ? 
+                     &Conf->XMLDataHooks : &Conf->XMLLeaveHooks;
+  UdmVarListReplaceStr(Vars,av[1],av[2]);
+  return UDM_OK;
+}
+
 static int srv_rpl_var(void *Cfg, size_t ac,char **av)
 {
   UDM_CFG  *C=(UDM_CFG*)Cfg;
@@ -1132,6 +1174,11 @@ static int env_rpl_num_var(void *Cfg, size_t ac,char **av)
   UDM_CFG  *C=(UDM_CFG*)Cfg;
   UDM_ENV  *Conf=C->Indexer->Conf;
   int  res=atoi(av[1]);
+  if(!strcasecmp(av[0], "DocSizeWeight"))
+  {
+    UdmVarListReplaceInt(&Conf->Vars, "MaxCoordFactor" ,res);
+    return UDM_OK;
+  }
   if(!strcasecmp(av[0],"MinWordLength"))Conf->WordParam.min_word_len=res;
   if(!strcasecmp(av[0],"MaxWordLength"))Conf->WordParam.max_word_len=res;
   UdmVarListReplaceInt(&Conf->Vars,av[0],res);
@@ -1201,6 +1248,7 @@ static UDM_CONFCMD commands[] =
 
   {"ImportEnv",            1,1,   env_rpl_env_var},
   {"DBAddr",               1,1,   env_rpl_var},
+  {"DefaultContentType",   1,1,   env_rpl_var},
   {"Listen",               1,1,   env_rpl_var},
   {"UseRemoteContentType", 1,1,   env_rpl_var},
   {"UseCRC32URLId",        1,1,   env_rpl_var},
@@ -1213,6 +1261,7 @@ static UDM_CONFCMD commands[] =
   {"GroupBySite",          1,1,   env_rpl_var},
   {"Cache",                1,1,   env_rpl_var},
   {"wf",                   1,1,   env_rpl_var},
+  {"StrictModeThreshold",  1,1,   env_rpl_var},
   {"PopRankSkipSameSite",  1,1,   env_rpl_var},
   {"PopRankFeedBack",      1,1,   env_rpl_var},
   {"PopRankUseTracking",   1,1,   env_rpl_var},
@@ -1234,9 +1283,21 @@ static UDM_CONFCMD commands[] =
   {"HlEnd",                1,1,   env_rpl_var},
   {"Log2stderr",           1,1,   env_rpl_var},
   {"PagesPerScreen",       1,1,   env_rpl_var},
+  {"SQLClearDBHook",       1,1,   env_rpl_var},
 #ifdef HAVE_SETVBUF
   {"StdoutBufferSize",     1,1,   env_rpl_var},
 #endif
+  {"AlwaysFoundWord",      1,1,   env_rpl_var},
+  {"CustomLog",            1,1,   env_rpl_var},
+  {"CurrentTime",          1,1,   env_rpl_var},
+  {"WordDistanceWeight",   1,1,   env_rpl_num_var},
+  {"MaxCoordFactor",       1,1,   env_rpl_num_var},
+  {"MinCoordFactor",       1,1,   env_rpl_num_var},
+  {"NumWordFactor",        1,1,   env_rpl_var},
+  {"URLDataThreshold",     1,1,   env_rpl_num_var},
+  {"DocSizeWeight",        1,1,   env_rpl_num_var},
+  {"RelevancyFactor",      1,1,   env_rpl_num_var},
+  {"DateFactor",           1,1,   env_rpl_num_var},
   {"MinWordLength",        1,1,   env_rpl_num_var},
   {"MaxWordLength",        1,1,   env_rpl_num_var},
   {"ExcerptSize",          1,1,   env_rpl_num_var},
@@ -1248,10 +1309,17 @@ static UDM_CONFCMD commands[] =
   {"Suggest",              1,1,   env_rpl_bool_var},
   {"IndexTime",            1,1,   env_rpl_bool_var},
   {"ExcerptStopword",      1,1,   env_rpl_bool_var},
+  {"UseCookie",            1,1,   env_rpl_bool_var},
+  {"UseNumericOperators",  1,1,   env_rpl_bool_var},
+  {"ReplaceVar",           2,2,   env_rpl_named_var},
 
   {"LocalCharset",         1,1,   env_rpl_charset},
   {"BrowserCharset",       1,1,   env_rpl_charset},
 
+  {"XMLEnterHook",         2,2,   rpl_xml_hook},
+  {"XMLLeaveHook",         2,2,   rpl_xml_hook},
+  {"XMLDataHook",          2,2,   rpl_xml_hook},
+  
   {"R0",                   1,1,   env_rpl_rand},
   {"R1",                   1,1,   env_rpl_rand},
   {"R2",                   1,1,   env_rpl_rand},
@@ -1267,6 +1335,9 @@ static UDM_CONFCMD commands[] =
   {"HTDBList",             1,1,   srv_rpl_var},
   {"HTDBDoc",              1,1,   srv_rpl_var},
   {"HTDBLimit",            1,1,   srv_rpl_var},
+  {"SQLImportSection",     1,1,   srv_rpl_var},
+  {"SQLExportHref",        1,1,   srv_rpl_var},
+  {"SQLWordForms",         1,1,   env_rpl_var},
   {"DefaultLang",          1,1,   srv_rpl_var},
   {"Category",             1,1,   srv_rpl_category},
   {"Tag",                  1,1,   srv_rpl_var},
@@ -1294,6 +1365,7 @@ static UDM_CONFCMD commands[] =
   {"MirrorHeadersRoot",    1,1,   srv_rpl_mirror},
   {"MirrorPeriod",         1,1,   srv_rpl_mirror},
   {"RemoteCharset",        1,1,   srv_rpl_charset},
+  {"RemoteFileNameCharset",1,1,   srv_rpl_charset},
   
   {"Disallow",             1,100, add_filter},
   {"Allow",                1,100, add_filter},
@@ -1301,9 +1373,11 @@ static UDM_CONFCMD commands[] =
   {"CheckMP3",             1,100, add_filter},
   {"CheckOnly",            1,100, add_filter},
   {"HrefOnly",             1,100, add_filter},
+  {"ImportOnly",           1,100, add_filter},
 
   {"IndexIf",              1,100, add_section_filter},
   {"NoIndexIf",            1,100, add_section_filter},
+  {"SkipIf",               1,100, add_section_filter},
   
   {"Server",               1,100, add_srv},
   {"Realm",                1,100, add_srv},
@@ -1314,8 +1388,8 @@ static UDM_CONFCMD commands[] =
   {"ReverseAlias",         1,100, add_alias},
   
   {"AddType",              1,100, add_type},
-  {"Mime",                 2,3,   add_parser},
-  {"Section",              3,8,   add_section},
+  {"Mime",                 2,4,   add_parser},
+  {"Section",              3,9,   add_section},
   {"Affix",                3,3,   add_affix},
   {"Spell",                3,3,   add_spell},
   {"StopwordFile",         1,1,   add_stoplist},
@@ -1324,7 +1398,7 @@ static UDM_CONFCMD commands[] =
   {"Synonym",              1,1,   add_synonym},
   {"LoadChineseList",      0,2,   add_chinese},
   {"LoadThaiList",         0,2,   add_thai},
-  {"Limit",                1,1,   add_limit},
+  {"Limit",                1,2,   add_limit},
   {"ServerTable",          1,1,   add_srv_table},
   {"FlushServerTable",     0,0,   flush_srv_table},
   {"DBListFree",           0,0,   dblist_free},
