@@ -98,12 +98,7 @@ void *UdmDBInit(void *vdb)
 #if HAVE_SQL
   UdmWordCacheInit(&db->WordCache);
 #endif
-  
-#if (HAVE_ODBC)
-  db->hDbc=SQL_NULL_HDBC;
-  db->hEnv=SQL_NULL_HENV;
-  db->hstmt=SQL_NULL_HSTMT;
-#endif
+ 
   return db;
 }
 
@@ -494,7 +489,11 @@ __C_LINK int __UDMCALL UdmURLAction(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd) {
 #ifdef HAVE_SQL
       default:
         res=UdmURLActionSQL(A,D,cmd,db);
-        if (cmd == UDM_URL_ACTION_EXPIRE) UDM_FREE(db->where);  /* clear db->where for next parameters */
+        if (cmd == UDM_URL_ACTION_EXPIRE)
+        {
+          UDM_FREE(db->where);  /* clear db->where for next parameters */
+          UDM_FREE(db->from);
+        }
         execflag = 1;
         break;
 #endif
@@ -655,11 +654,6 @@ int UdmFindWordsDB(UDM_AGENT *A, UDM_RESULT *CurRes, UDM_DB *db)
       break;
 #endif
   }
-  if (!CurRes->PerSite)
-  {
-    size_t nbytes= sizeof(size_t) * CurRes->total_found;
-    CurRes->PerSite= nbytes ? (size_t*)UdmXmalloc(nbytes) : NULL;
-  }
   return rc;
 }
 
@@ -681,6 +675,102 @@ void *SearchThread(void *arg)
   return NULL;
 }
 
+
+static int
+UdmResultSetMachineNumber(UDM_RESULT *Res, size_t num)
+{
+  size_t i;
+  UDM_URL_CRD *Coords= Res->CoordList.Coords;
+
+  for(i= 0; i < Res->total_found; i++)
+  {
+    /* 
+      We use (256-i) to sort a document from the first database
+      before the same document from the second database.
+    */
+    Coords[i].coord = (Coords[i].coord << 8) + (255 - (num & 255));
+   }
+  return UDM_OK;
+}
+
+
+static int
+UdmResultJoin(UDM_AGENT *A,
+              UDM_RESULT *TmpRes, size_t dbfrom, size_t dbto,
+              UDM_RESULT *Res)
+{
+  size_t i;
+  unsigned long ticks;
+  
+  for (Res->total_found= 0, i= dbfrom; i < dbto; i++)
+    Res->total_found+= TmpRes[i].total_found;
+
+  Res->num_rows= Res->CoordList.ncoords= Res->total_found;
+  if (dbto - dbfrom == 1)
+  {
+    Res->CoordList.Coords= TmpRes[dbfrom].CoordList.Coords;
+    Res->CoordList.Data= TmpRes[dbfrom].CoordList.Data;
+    UdmResultSetMachineNumber(Res, dbfrom);
+    return UDM_OK;
+  }
+
+  ticks= UdmStartTimer();
+  UdmLog(A, UDM_LOG_DEBUG, "Start joining results from %d dbs", dbto - dbfrom);
+
+  if (Res->total_found > 0)
+  {
+    size_t offs;
+
+    Res->CoordList.Coords= (UDM_URL_CRD*)UdmMalloc(sizeof(UDM_URL_CRD)
+                                                   * Res->total_found);
+    Res->CoordList.Data= (UDM_URLDATA*)UdmMalloc(sizeof(UDM_URLDATA)
+                                                 * Res->total_found);
+    for (offs= 0, i= dbfrom; i < dbto; offs+= TmpRes[i].total_found, i++)
+    {
+      UDM_RESULT  *CurRes= &TmpRes[i];
+      UDM_URL_CRD *CurCoords= TmpRes[i].CoordList.Coords;
+      if(CurCoords)
+      {
+        UdmResultSetMachineNumber(CurRes, i);
+        memcpy(Res->CoordList.Coords + offs,CurCoords,
+               sizeof(CurCoords[0])*CurRes->total_found);
+        if (CurRes->CoordList.Data != NULL)
+        {
+          memcpy(Res->CoordList.Data + offs, CurRes->CoordList.Data,
+                 sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
+        }
+        else
+        {
+          bzero(Res->CoordList.Data + offs,
+                sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
+        }
+      }
+      UDM_FREE(CurRes->CoordList.Coords);
+      UDM_FREE(CurRes->CoordList.Data);
+    }
+  }
+
+  if (dbto - dbfrom > 1)
+  {
+    int use_site_id = ((!strcasecmp(UdmVarListFindStr(&A->Conf->Vars, "GroupBySite", "no"), "yes"))
+                        && (UdmVarListFindInt(&A->Conf->Vars, "site", 0) == 0));
+    if (use_site_id)
+    {
+      UdmSortSearchWordsBySite(&Res->CoordList, Res->CoordList.ncoords);
+      UdmGroupBySite(A, Res);
+    }
+    UdmSortSearchWordsByPattern(Res, &Res->CoordList, Res->CoordList.ncoords,
+                                UdmVarListFindStr(&A->Conf->Vars, "s", "RP"));
+    Res->num_rows= Res->total_found= Res->CoordList.ncoords;
+  }
+
+  ticks= UdmStartTimer() - ticks;
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  joining results:\t%.2f", (float)ticks / 1000);
+
+  return UDM_OK;
+}
+
+
 int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res)
 {
   const char  *cache_mode= UdmVarListFindStr(&A->Conf->Vars, "Cache", "no");
@@ -716,7 +806,10 @@ int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res)
         search_param[i].thd= thd;
       }
       else
-        UdmFindWordsDB(A, CurRes, db);
+      {
+        if (UDM_OK != (res= UdmFindWordsDB(A, CurRes, db)))
+          return res;
+      }
     }
     
     if (use_threads)
@@ -734,72 +827,8 @@ int UdmFindWords(UDM_AGENT *A, UDM_RESULT *Res)
       for (i= dbfrom; i < dbto; i++)
         UdmAgentFree(search_param[i].Agent);
     }
-    
-    for (Res->total_found= 0, i= dbfrom; i < dbto; i++)
-      Res->total_found+= TmpRes[i].total_found;
 
-    if (Res->total_found > 0)
-    {
-      size_t offs;
-
-      Res->CoordList.Coords= (UDM_URL_CRD*)UdmMalloc(sizeof(UDM_URL_CRD)
-                                                     * Res->total_found);
-      Res->CoordList.Data= (UDM_URLDATA*)UdmMalloc(sizeof(UDM_URLDATA)
-                                                   * Res->total_found);
-      Res->PerSite= (size_t*)UdmMalloc(sizeof(size_t) * Res->total_found);
-      for (offs= 0, i= dbfrom; i < dbto; offs+= TmpRes[i].total_found, i++)
-      {
-        UDM_RESULT  *CurRes= &TmpRes[i];
-        UDM_URL_CRD *CurCoords= TmpRes[i].CoordList.Coords;
-        if(CurCoords)
-        {
-          size_t j;
-          
-          /* Set machine number */
-          for(j= 0; j < CurRes->total_found; j++)
-          {
-            /* 
-              We use (256-i) to sort a document from the first database
-              before the same document from the second database.
-            */
-            CurCoords[j].coord = (CurCoords[j].coord << 8) + (255 - (i & 255));
-          }
-          
-          memcpy(Res->CoordList.Coords + offs,CurCoords,
-                 sizeof(CurCoords[0])*CurRes->total_found);
-          memcpy(Res->PerSite + offs, CurRes->PerSite,
-                 sizeof(size_t) * CurRes->total_found);
-          if (CurRes->CoordList.Data != NULL)
-          {
-            memcpy(Res->CoordList.Data + offs, CurRes->CoordList.Data,
-                   sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
-          }
-          else
-          {
-            bzero(Res->CoordList.Data + offs,
-                  sizeof(Res->CoordList.Data[0]) * CurRes->total_found);
-          }
-        }
-        UDM_FREE(CurRes->CoordList.Coords);
-        UDM_FREE(CurRes->CoordList.Data);
-        UDM_FREE(CurRes->PerSite);
-      }
-    }
-
-    Res->num_rows= Res->CoordList.ncoords= Res->total_found;
-    
-    if (dbto - dbfrom > 1)
-    {
-      int use_site_id = ((!strcasecmp(UdmVarListFindStr(&A->Conf->Vars, "GroupBySite", "no"), "yes"))
-                          && (UdmVarListFindInt(&A->Conf->Vars, "site", 0) == 0));
-      if (use_site_id)
-      {
-        UdmSortSearchWordsBySite(&Res->CoordList, Res->CoordList.ncoords);
-        UdmGroupBySite(A, Res);
-      }
-      UdmSortSearchWordsByPattern(Res, &Res->CoordList, Res->CoordList.ncoords, UdmVarListFindStr(&A->Conf->Vars, "s", "RP"));
-      Res->total_found = Res->CoordList.ncoords;
-    }
+    UdmResultJoin(A, TmpRes, dbfrom, dbto, Res);
 
     if (ResultsLimit > 0 && ResultsLimit < Res->total_found)
     {
@@ -822,10 +851,11 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   UDM_RESULT  *Res;
   int    res=UDM_OK;
   unsigned long  ticks=UdmStartTimer(), ticks_;
-  size_t i, dbfrom = 0, dbto =  A->Conf->dbl.nitems;
-  int    page_number = UdmVarListFindInt(&A->Conf->Vars, "np", 0);
-  int    page_size   = UdmVarListFindInt(&A->Conf->Vars, "ps", 10);
-  size_t  ExcerptSize = (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptSize", 256);
+  size_t  i, dbfrom= 0, dbto=  A->Conf->dbl.nitems;
+  size_t  page_number= (size_t) UdmVarListFindInt(&A->Conf->Vars, "np", 0);
+  size_t  page_size=   (size_t) UdmVarListFindInt(&A->Conf->Vars, "ps", 10);
+  size_t  offs=        (size_t) UdmVarListFindInt(&A->Conf->Vars, "offs", 0);
+  size_t  ExcerptSize= (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptSize", 256);
   size_t  ExcerptPadding = (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptPadding", 40);
   char    str[128];
 
@@ -833,6 +863,8 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   
   Res=UdmResultInit(NULL);
   UdmPrepare(A, Res);
+  if (!Res->WWList.nwords)
+    goto ret;
   
   UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
 
@@ -853,13 +885,13 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   UdmVarListDel(&A->Conf->Vars, "orig_m");
   UdmVarListReplaceInt(&A->Conf->Vars, "CurrentTimestamp", (int) time(0));
 
-  Res->first = page_number * page_size;  
+  Res->first = offs ? offs : page_number * page_size;  
   if(Res->first >= Res->total_found)
   {
     /*Res->first = (Res->total_found)? (Res->total_found - 1) : 0;*/
     Res->last= Res->first;
     Res->num_rows= 0;
-    goto ret;
+    goto conv; /* jump to converting variables into BrowserCharset */
   }
   if((Res->first+page_size)>Res->total_found)
   {
@@ -884,10 +916,8 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
     UdmVarListReplaceStr(&Res->Doc[i].Sections, "Score", str);
     UdmVarListReplaceInt(&Res->Doc[i].Sections,"Order",(int)(i + Res->first + 1));
     UdmVarListReplaceInt(&Res->Doc[i].Sections, "dbnum", 255 - (int)(score & 0xFF));
-    if (Res->PerSite)
-    {
-      UdmVarListReplaceUnsigned(&Res->Doc[i].Sections, "PerSite", Res->PerSite[i + Res->first]);
-    }
+    UdmVarListReplaceUnsigned(&Res->Doc[i].Sections, "PerSite",
+                              Res->CoordList.Data[i + Res->first].per_site);
   }
   
   for (i = dbfrom; i < dbto; i++)
@@ -938,6 +968,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   ticks_ = UdmStartTimer() - ticks_;
   UdmLog(A, UDM_LOG_DEBUG, "Stop  Order:\t\t\t%.2f", (float)ticks_/1000);
   
+conv:
   ticks_=UdmStartTimer();
   UdmLog(A, UDM_LOG_DEBUG, "Start UdmConvert");
   UdmConvert(A->Conf, Res, A->Conf->lcs, A->Conf->bcs);
@@ -979,7 +1010,6 @@ static int UdmStr2DBMode(const char * str1)
   if(!strncasecmp(str1,"single",6))m=UDM_DBMODE_SINGLE;
   else if(!strncasecmp(str1,"multi",5))m=UDM_DBMODE_MULTI;
   else if(!strncasecmp(str1,"blob",4))m=UDM_DBMODE_BLOB;
-  else if(!strncasecmp(str1,"fly",3))m=UDM_DBMODE_FLY;
   return(m);
 }
 
@@ -998,6 +1028,7 @@ __C_LINK const char* __UDMCALL UdmDBTypeToStr(int dbtype)
     case UDM_DB_ACCESS:  return "access";
     case UDM_DB_DB2:     return "db2";
     case UDM_DB_CACHE:   return "cache";
+    case UDM_DB_SYBASE:  return "sybase";
   }
   return "unknown_dbtype";
 }
@@ -1010,7 +1041,6 @@ __C_LINK const char* __UDMCALL UdmDBModeToStr(int dbmode)
     case UDM_DBMODE_SINGLE:  return "single";
     case UDM_DBMODE_MULTI:   return "multi";
     case UDM_DBMODE_BLOB:    return "blob";
-    case UDM_DBMODE_FLY:     return "fly";
   }
   return "unknown_dbmode";
 }
@@ -1054,33 +1084,42 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
 #if (HAVE_ORACLE8)
   {
     "oracle8", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_oracle_handler
   },
   {
     "oracle", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_oracle_handler
   },
 #endif
 #if (HAVE_CTLIB)
   {
     "mssql", UDM_DB_MSSQL, UDM_DB_MSSQL, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    &udm_sqldb_ctlib_handler
+  },
+  {
+    "sybase", UDM_DB_SYBASE, UDM_DB_MSSQL, 1,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_ctlib_handler
   },
 #endif
 #if (HAVE_MYSQL)
   { 
     "mysql", UDM_DB_MYSQL, UDM_DB_MYSQL, 1,
-    UDM_SQL_HAVE_LIMIT|UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_mysql_handler
   },
 #endif
 #if (HAVE_PGSQL)
   {
     "pgsql", UDM_DB_PGSQL, UDM_DB_PGSQL, 1,
-    UDM_SQL_HAVE_LIMIT|UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT,
     &udm_sqldb_pgsql_handler,
   },
 #endif
@@ -1101,69 +1140,80 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sqlite",
     UDM_DB_SQLITE, UDM_DB_SQLITE, 1,
-    UDM_SQL_HAVE_LIMIT|UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY,
     &udm_sqldb_sqlite_handler,
   },
 #endif
 #if (HAVE_ODBC)
   {
     "odbc-solid", UDM_DB_SOLID, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-sapdb", UDM_DB_SAPDB, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-db2", UDM_DB_DB2, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_STDHEX,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-access", UDM_DB_ACCESS, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mimer", UDM_DB_MIMER, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-cache", UDM_DB_CACHE, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   { 
     "odbc-virtuoso", UDM_DB_VIRT, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle8", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mssql", UDM_DB_MSSQL, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_TRUNCATE,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    &udm_sqldb_odbc_handler,
+  },
+  {
+    "odbc-sybase", UDM_DB_SYBASE, UDM_DB_ODBC, 1,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mysql", UDM_DB_MYSQL, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_LIMIT|UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-pgsql", UDM_DB_PGSQL, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_LIMIT|UDM_SQL_HAVE_GROUPBY|UDM_SQL_HAVE_SUBSELECT,
+    UDM_SQL_HAVE_LIMIT      | UDM_SQL_HAVE_GROUPBY |
+    UDM_SQL_HAVE_SUBSELECT  | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1174,7 +1224,7 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     are more then 1500. We'd better have to fix code to avoid 
     big INs instead of hidding DBSQL_IN.
     */
-    UDM_SQL_HAVE_GROUPBY,
+    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND,
     &udm_sqldb_odbc_handler,
   },
 #endif
@@ -1308,10 +1358,12 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
       return UDM_ERROR;
     if (DBMode == UDM_DBMODE_BLOB  &&
         db->DBType != UDM_DB_MYSQL &&
+        db->DBType != UDM_DB_SYBASE &&
         db->DBType != UDM_DB_MSSQL &&
         db->DBType != UDM_DB_MIMER &&
         db->DBType != UDM_DB_ORACLE8 &&
-        db->DBType != UDM_DB_DB2)
+        db->DBType != UDM_DB_DB2 &&
+        db->DBType != UDM_DB_PGSQL)
       return UDM_ERROR;
   }
 
@@ -1571,27 +1623,20 @@ int UdmWordCacheFlush (UDM_AGENT *Indexer)
 {
 #ifdef HAVE_SQL
   size_t i;
-
-  for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
+  for (i= 0; i < Indexer->Conf->dbl.nitems; i++)
   {
-    int rc;
-    UDM_DB *db = &Indexer->Conf->dbl.db[i];
+    int rc= UDM_OK;
+    UDM_DB *db= &Indexer->Conf->dbl.db[i];
     UDM_GETLOCK(Indexer, UDM_LOCK_DB);
-    if (db->DBMode == UDM_DBMODE_MULTI)
-    {
-      if (UDM_OK != (rc= UdmWordCacheWrite(Indexer, db, 0)))
-      {
-        UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr);
-        return rc;
-      }
+    switch (db->DBMode) {
+      case UDM_DBMODE_MULTI:
+        rc= UdmWordCacheWrite(Indexer, db, 0);
+        break;
     }
-    if (db->DBMode == UDM_DBMODE_FLY)
+    if (rc != UDM_OK)
     {
-      if (UDM_OK != (rc= UdmDBModeFlyWrite(Indexer, db, 0)))
-      {
-        UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr);
-        return rc;
-      }
+      UdmLog(Indexer, UDM_LOG_ERROR, "%s", db->errstr);
+      return rc;
     }
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
   }
@@ -1617,8 +1662,8 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
       rc= UdmMulti2BlobSQL(Indexer, db);
     else if (db->DBMode == UDM_DBMODE_SINGLE)
       rc= UdmSingle2BlobSQL(Indexer, db);
-    else if (db->DBMode == UDM_DBMODE_FLY)
-      rc= UdmDBModeFlyMerge(Indexer, db);
+    else if (db->DBMode == UDM_DBMODE_BLOB)
+      rc= UdmBlob2BlobSQL(Indexer, db);
     UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
     if (rc != UDM_OK)
     {
@@ -1633,6 +1678,69 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
   return UDM_OK;
 }
 
+
+int UdmRewriteURL(UDM_AGENT *Indexer)
+{
+#ifdef HAVE_SQL
+  size_t i;
+  unsigned long ticks;
+
+  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob");
+  ticks=UdmStartTimer();
+
+  for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
+  {
+    int rc;
+    UDM_DB *db = &Indexer->Conf->dbl.db[i];
+    UDM_GETLOCK(Indexer, UDM_LOCK_DB);
+    rc= UdmBlobWriteURL(Indexer, db, "bdict",
+          UdmVarListFindBool(&db->Vars, "deflate", 0));
+    UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
+    if (rc != UDM_OK)
+    {
+      UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr); 
+      return rc;
+    }
+  }
+
+  ticks=UdmStartTimer()-ticks;
+  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob finished\t%.2f",(float)ticks/1000);
+#endif
+  return UDM_OK;
+}
+
+
+int UdmRewriteLimits(UDM_AGENT *Indexer)
+{
+#ifdef HAVE_SQL
+  size_t i;
+  unsigned long ticks;
+
+  UdmLog(Indexer,UDM_LOG_ERROR,"Rewritting limits");
+  ticks=UdmStartTimer();
+
+  for (i = 0; i < Indexer->Conf->dbl.nitems; i++)
+  {
+    int rc;
+    UDM_DB *db = &Indexer->Conf->dbl.db[i];
+    UDM_GETLOCK(Indexer, UDM_LOCK_DB);
+    rc= UdmBlobWriteLimits(Indexer, db, "bdict",
+          UdmVarListFindBool(&db->Vars, "deflate", 0));
+    UDM_RELEASELOCK(Indexer, UDM_LOCK_DB);
+    if (rc != UDM_OK)
+    {
+      UdmLog(Indexer,UDM_LOG_ERROR,"%s",db->errstr); 
+      return rc;
+    }
+  }
+
+  ticks=UdmStartTimer()-ticks;
+  UdmLog(Indexer,UDM_LOG_ERROR,"Rewritting limits\t%.2f",(float)ticks/1000);
+#endif
+  return UDM_OK;
+}
+
+
 int UdmExport (UDM_AGENT *Indexer)
 {
 #ifdef HAVE_SQL
@@ -1889,7 +1997,7 @@ static int bccmpwrd (UDM_BLOB_CACHE_WORD *s1, UDM_BLOB_CACHE_WORD *s2)
 
 void UdmBlobCacheSort (UDM_BLOB_CACHE *cache)
 {
-  qsort(cache->words, cache->nwords, sizeof(UDM_BLOB_CACHE_WORD), (qsort_cmp)bccmpwrd);
+  qsort(cache->words, cache->nwords, sizeof(UDM_BLOB_CACHE_WORD), (udm_qsort_cmp)bccmpwrd);
 }
 
 UDM_WORD_CACHE *UdmWordCacheInit (UDM_WORD_CACHE *cache)
@@ -1934,21 +2042,23 @@ void UdmWordCacheFree (UDM_WORD_CACHE *cache)
 
 static int wccmpwrd (UDM_WORD_CACHE_WORD *s1, UDM_WORD_CACHE_WORD *s2)
 {
-  int _ = s1->seed - s2->seed;
-  if (! _) _ = strcmp(s1->word, s2->word);
-  if (! _) _ = s1->secno - s2->secno;
-  if (! _) {
-    if (s1->url_id > s2->url_id) _ = 1;
-    else if (s1->url_id < s2->url_id) _ = -1;
-    else _ = 0;
-  }
-  if (! _) _ = s1->coord - s2->coord;
-  return(_);
+  register int _;
+  if ((_= s1->seed - s2->seed))
+    return _;
+  if ((_= s1->secno - s2->secno))
+    return _;
+  if (s1->url_id > s2->url_id)
+    return 1;
+  if (s1->url_id < s2->url_id)
+    return -1;
+  if ((_= strcmp(s1->word, s2->word)))
+    return _;
+  return s1->coord - s2->coord;
 }
 
 void UdmWordCacheSort (UDM_WORD_CACHE *cache)
 {
-  UdmSort(cache->words, cache->nwords, sizeof(UDM_WORD_CACHE_WORD), (qsort_cmp)wccmpwrd);
+  UdmSort(cache->words, cache->nwords, sizeof(UDM_WORD_CACHE_WORD), (udm_qsort_cmp)wccmpwrd);
 }
 
 int UdmWordCacheAdd (UDM_WORD_CACHE *cache, urlid_t url_id, const char *word, int intag)
