@@ -77,6 +77,23 @@ void __UDMCALL UdmDocFree(UDM_DOCUMENT *Result){
 	}
 }
 
+
+void UdmURLCRDListListInit(UDM_URLCRDLISTLIST *Lst)
+{
+  bzero((void*)Lst, sizeof(*Lst));
+}
+
+int UdmURLCRDListListAdd(UDM_URLCRDLISTLIST *LstLst, UDM_URLCRDLIST *List)
+{
+  size_t nbytes= (LstLst->nlists+1) * sizeof(UDM_URLCRDLIST);
+  if (!(LstLst->List= (UDM_URLCRDLIST*) UdmRealloc(LstLst->List, nbytes)))
+    return UDM_ERROR;
+  LstLst->List[LstLst->nlists]= *List;
+  LstLst->nlists++;
+  return UDM_OK;
+}
+
+
 void UdmURLCRDListListFree(UDM_URLCRDLISTLIST *Lst){
 	size_t i;
 	
