@@ -77,7 +77,6 @@ __C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
 #ifdef MECAB
         mecab_destroy (Env->mecab);
 #endif
-	UdmDBListFree(&Env->dbl);
 	UdmRobotListFree(&Env->Robots);
 	UdmHrefListFree(&Env->Hrefs);
 	UdmResultFree(&Env->Targets);
@@ -90,9 +89,16 @@ __C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
 	UdmMatchListFree(&Env->ReverseAliases);
 	UdmMatchListFree(&Env->Filters);
 	UdmMatchListFree(&Env->SectionFilters);
-	
+	UdmMatchListFree(&Env->SectionMatch);
+	UdmMatchListFree(&Env->SectionHdrMatch);
+	UdmMatchListFree(&Env->SectionGsrMatch);
+		
 	UdmSynonymListFree(&Env->Synonyms);
 	UdmVarListFree(&Env->Sections);
+	UdmVarListFree(&Env->XMLEnterHooks);
+	UdmVarListFree(&Env->XMLLeaveHooks);
+	UdmVarListFree(&Env->XMLDataHooks);
+	UdmVarListFree(&Env->Cookies);
 	UdmLangMapListSave(&Env->LangMaps);
 	UdmLangMapListFree(&Env->LangMaps);
 	UdmServerListFree(&Env->Servers);
@@ -101,7 +107,7 @@ __C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
 	UdmVarListFree(&Env->Vars);
 	UdmChineseListFree(&Env->Chi);
 	UdmChineseListFree(&Env->Thai);
-
+	UdmDBListFree(&Env->dbl);
 	if(Env->freeme)UDM_FREE(Env);
 }
 
