@@ -62,10 +62,13 @@ static UDM_CTMATCH udm_magic[]=
   {0, 30, "<!--",           4,  "text/html"},
   {0, 30, "<H1",            3,  "text/html"},
   {0, 30, "<h1",            3,  "text/html"},
-  {0, 30, NULL, NULL}
+  {0, 30, "<BASE",          5,  "text/html"},
+  {0, 30, "<base",          5,  "text/html"},
+  {0, 30, NULL, 0, NULL}
 };
 
-const char *UdmGuessContentType(const char *begin, size_t length)
+const char *UdmGuessContentType(const char *begin, size_t length,
+                                const char *default_type)
 {
   UDM_CTMATCH *m;
   const char *beg, *end, *cur;
@@ -81,7 +84,10 @@ const char *UdmGuessContentType(const char *begin, size_t length)
         return m->content_type;
     }
   }
-  
+
+  if (default_type)
+    return default_type;
+
   /* Check if text/plain */
   end= length > 128 ? begin + 128 : begin + length;
   for (cur= begin; cur < end; cur++)
@@ -182,7 +188,7 @@ __C_LINK int __UDMCALL UdmLoadLangMapFile(UDM_LANGMAPLIST *L, const char * filen
                }
                if (Cmap == NULL) {
 		 Cmap = FindLangMap(L, Clanguage, Ccharset, filename);
-		 UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpIndex);
+		 UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex);
 	       }
                if (Cmap == NULL) return UDM_ERROR;
                *s='\0';
@@ -243,7 +249,7 @@ void UdmLangMapListSave(UDM_LANGMAPLIST *List) {
          fprintf(out, "Language: %s\n", Cmap->lang);
          fprintf(out, "Charset:  %s\n", Cmap->charset);
          fprintf(out, "\n\n");
-         UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpCount);
+         UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpCount);
 	 minv = (Cmap->memb[UDM_LM_TOPCNT - 1].count > 1000) ? 1000 : Cmap->memb[UDM_LM_TOPCNT - 1].count;
          for(j = 0; j < UDM_LM_TOPCNT; j++) {
            Cmap->memb[j].count -= Cmap->memb[UDM_LM_TOPCNT - 1].count - minv;
@@ -342,8 +348,8 @@ int UdmLMcmpIndex(UDM_LANGITEM *m1, UDM_LANGITEM *m2) {
 
 void UdmPrepareLangMap(UDM_LANGMAP * map){
 
-     UdmSort(map->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpCount);
-     UdmSort(map->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpIndex);
+     UdmSort(map->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpCount);
+     UdmSort(map->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex);
 }
 
 
@@ -353,7 +359,7 @@ void UdmCheckLangMap(UDM_LANGMAP * map0, UDM_LANGMAP * map1, UDM_MAPSTAT *stat,
 
      stat->hits = stat->miss = 0;
      for (i = 0; i < UDM_LM_TOPCNT; i++) {
-       if ( (HIT = bsearch(&map1->memb[i], map0->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpIndex)) == NULL) {
+       if ( (HIT = bsearch(&map1->memb[i], map0->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex)) == NULL) {
 	 stat->miss++;
        } else {
 	 stat->hits += UDM_LM_TOPCNT - abs(HIT - map0->memb);
@@ -383,10 +389,28 @@ int  UdmGuessCharSet(UDM_AGENT *Indexer, UDM_DOCUMENT * Doc,UDM_LANGMAPLIST *Lis
      update_lm = !strcasecmp(UdmVarListFindStr(&Indexer->Conf->Vars, "LangMapUpdate", "no"), "yes");
      UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
 
-     if (*charset == '\0') {
-       if (*server_charset != '\0' 
-	   && (!use_meta || *meta_charset == '\0' || !strcasecmp(server_charset, meta_charset) )) charset = server_charset;
-       else if (use_meta && *server_charset == '\0' && *meta_charset != '\0') charset = meta_charset;
+     if (!List->nmaps)
+     {
+       /*
+         Choose the first non-empty charset in this order:
+         - server_charset, i.e. the one from HTTP headers
+         - meta_charset, from <META NAME="Content-Type">,
+           or <?xml encoding=""?>
+         - RemoteCharset, the default one for this server
+       */
+       if (*server_charset)
+         charset= server_charset;
+       else if (use_meta && *meta_charset)
+         charset= meta_charset;
+     }
+     else if (*charset == '\0')
+     {
+       if (*server_charset != '\0' && (!use_meta ||
+                                       *meta_charset == '\0' ||
+                                       !strcasecmp(server_charset, meta_charset) ))
+         charset= server_charset;
+       else if (use_meta && *server_charset == '\0' && *meta_charset != '\0')
+         charset= meta_charset;
      }
      UdmVarListReplaceStr(&Doc->Sections, "Charset", charset);
 
@@ -507,7 +531,7 @@ int  UdmGuessCharSet(UDM_AGENT *Indexer, UDM_DOCUMENT * Doc,UDM_LANGMAPLIST *Lis
 
 	 UDM_GETLOCK(Indexer,UDM_LOCK_CONF);
          if ((Cmap = FindLangMap(&Indexer->Conf->LangMaps, lang, charset, NULL)) != NULL) {
-	   UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpIndex);
+	   UdmSort(Cmap->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex);
            for (i = 0; i < UDM_LM_HASHMASK+1; i++) {
 	     if (Cmap->memb[i].count == 0) {
 	       strcpy(Cmap->memb[i].str, LangMap->memb[i].str);
