@@ -62,7 +62,7 @@ static int host_addr_add(UDM_HOSTLIST *List,const char *hostname, struct in_addr
 	
 	if((List->nhost_addr>=List->mhost_addr)&&(List->mhost_addr<MAX_HOST_ADDR_SIZE)){
 		List->mhost_addr+=32;
-		List->host_addr = realloc(List->host_addr, List->mhost_addr*sizeof(UDM_HOST_ADDR));
+		List->host_addr = UdmRealloc(List->host_addr, List->mhost_addr*sizeof(UDM_HOST_ADDR));
 		bzero((void*)&List->host_addr[List->nhost_addr], (List->mhost_addr-List->nhost_addr)*sizeof(UDM_HOST_ADDR));
 	}
 
@@ -87,7 +87,7 @@ static int host_addr_add(UDM_HOSTLIST *List,const char *hostname, struct in_addr
 	if(addr)memcpy(&List->host_addr[min_id].addr, addr, sizeof( struct in_addr));
 	
 	UDM_FREE(List->host_addr[min_id].hostname);
-	List->host_addr[min_id].hostname = (char*)strdup(hostname);
+	List->host_addr[min_id].hostname = (char*)UdmStrdup(hostname);
 	List->host_addr[min_id].net_errors=0;
 
 	UdmSort(List->host_addr, List->nhost_addr, sizeof(UDM_HOST_ADDR), cmphost);
