@@ -189,6 +189,7 @@ __C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT * Indexer)
         }
       }
 
+      UdmVarListAddLst(&Doc.Sections, &H->Vars, NULL, "*");
       UdmVarListReplaceInt(&Doc.Sections, "Referrer-ID", H->referrer);
       UdmVarListReplaceUnsigned(&Doc.Sections,"Hops", H->hops);
       UdmVarListReplaceStr(&Doc.Sections,"URL",H->url?H->url:"");
@@ -198,6 +199,7 @@ __C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT * Indexer)
       UdmVarListReplaceInt(&Doc.Sections, "HTDB_URL_ID", H->rec_id);
       if(UDM_OK != (rc= UdmURLAction(Indexer, &Doc, action)))
         goto ret;
+      UdmVarListFree(&Doc.Sections);
       H->stored= 1;
     }
   }
@@ -337,23 +339,59 @@ static int UdmFilterFind(UDM_MATCHLIST *L,const char *newhref,char *reason){
   return res;
 }
 
+
+static UDM_MATCH *
+UdmMatchSectionListFindWithSubst(UDM_MATCHLIST *L, UDM_DOCUMENT *Doc,
+                                 size_t nparts, UDM_MATCH_PART *Parts)
+{
+  UDM_DSTR dstr;
+  size_t i;
+  UdmDSTRInit(&dstr, 128);
+  for(i=0;i<L->nmatches;i++)
+  {
+    UDM_MATCH *M=&L->Match[i];
+    const char *src;
+    if (strchr(M->section, '$'))
+    {
+      UdmDSTRReset(&dstr);
+      UdmDSTRParse(&dstr, M->section, &Doc->Sections);
+      src= dstr.data;
+    }
+    else
+    {
+      src= UdmVarListFindStr(&Doc->Sections, M->section, "");
+    }
+    if(!UdmMatchExec(M, src, src, nparts, Parts))
+    {
+      UdmDSTRFree(&dstr);
+      return M;
+    }
+  }
+  UdmDSTRFree(&dstr);
+  return NULL;
+}
+
+
 static int UdmSectionFilterFind(UDM_MATCHLIST *L, UDM_DOCUMENT *Doc, char *reason){
   UDM_MATCH_PART  P[NS];
   UDM_MATCH  *M;
   int    res=UDM_METHOD_INDEX;
-  
 #ifdef USE_PARANOIA
   void *paran = UdmViolationEnter();
 #endif
 
-  if((M=UdmMatchSectionListFind(L, Doc, NS, P))) {
-    sprintf(reason,"%s %s %s %s '%s'", M->arg,
+  if((M=UdmMatchSectionListFindWithSubst(L, Doc, NS, P)))
+  {
+    sprintf(reason,"%s %s %s %s '%s' '%s'", M->arg,
             M->nomatch ? "NoMatch" : "Match",
             UdmMatchTypeStr(M->match_type),
             M->case_sense ? "Sensitive" : "InSensitive",
+            M->section,
             M->pattern);
-    res=UdmMethod(M->arg);
-  }else{
+    res= UdmMethod(M->arg);
+  }
+  else
+  {
     sprintf(reason,"Allow by default");
   }
   UdmViolationExit(paran);
@@ -461,6 +499,14 @@ int UdmConvertHref(UDM_AGENT *Indexer, UDM_URL *CurURL,
   }
   
   Href->max_doc_per_site= UdmVarListFindInt(&Srv->Vars, "MaxDocPerSite", 0);
+  
+  {
+    const char *val= UdmVarListFindStr(&Srv->Vars, "SQLExportHref", NULL);
+    if (val)
+    {
+      UdmVarListAddStr(&Href->Vars, "SQLExportHref", val);
+    }
+  }
 
   /* FIXME: add MaxHops, Robots */
   UDM_FREE(Href->url);
@@ -843,6 +889,16 @@ static int UdmDocProcessResponseHeaders(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
       }
     }
   }
+  if (UdmVarListFindBool(&Indexer->Conf->Vars, "UseCookie", 0))
+  {
+    size_t i;
+    for (i= 0; i < Doc->Sections.nvars; i++)
+    {
+      var= &Doc->Sections.Var[i];
+      if (!strncmp(var->name, "Set-Cookie.", 11))
+        UdmVarListReplaceStr(&Indexer->Conf->Cookies, var->name + 11, var->val);
+    }
+  }
   UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
   if(!UdmVarListFind(&Doc->Sections,"Content-Type")) {
     UdmVarListAddStr(&Doc->Sections,"Content-Type","application/octet-stream");
@@ -946,7 +1002,7 @@ ex:
 }
 #endif
 
-static size_t UdmDSTRParse(UDM_DSTR *d, const char *fmt, UDM_VARLIST *L)
+size_t UdmDSTRParse(UDM_DSTR *d, const char *fmt, UDM_VARLIST *L)
 {
   for ( ; fmt[0]; fmt++)
   {
@@ -974,13 +1030,16 @@ static size_t UdmDSTRParse(UDM_DSTR *d, const char *fmt, UDM_VARLIST *L)
 }
 
 
-static int UdmParseSections (UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
+static int
+UdmParseSections(UDM_AGENT *Indexer,
+                 UDM_MATCHLIST *SectionMatch,
+                 UDM_DOCUMENT *Doc, UDM_CHARSET *doccs)
 {
   size_t i;
   UDM_DSTR tbuf;
 
   UdmDSTRInit(&tbuf, 1024);
-  for (i = 0; i < Indexer->Conf->SectionMatch.nmatches; i++)
+  for (i = 0; i < SectionMatch->nmatches; i++)
   {
     UDM_VAR *Sec;
     UDM_HTMLTOK tag;
@@ -989,7 +1048,7 @@ static int UdmParseSections (UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
     UDM_MATCH_PART  Parts[10];
     size_t nparts = 10;
     UDM_TEXTITEM Item;
-    UDM_MATCH *Alias= &Indexer->Conf->SectionMatch.Match[i];
+    UDM_MATCH *Alias= &SectionMatch->Match[i];
 
     if (Alias->arg1)
     {
@@ -1011,15 +1070,7 @@ static int UdmParseSections (UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
     Item.href=NULL;
     Item.section=Sec->section;
     Item.section_name=Sec->name;
-
-/*
-    Item.str = udm_strtok_r(buf, "\r\n", &lt);
-    while(Item.str)
-    {
-      UdmTextListAdd(&Doc->TextList,&Item);
-      Item.str = udm_strtok_r(NULL, "\r\n", &lt);
-    }
-*/
+    Item.flags= 0;
 
     UdmHTMLTOKInit(&tag);
     for (htok= UdmHTMLToken(buf, &last, &tag); htok; 
@@ -1027,10 +1078,35 @@ static int UdmParseSections (UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
     {
       if (tag.type == UDM_HTML_TXT && ! tag.script && ! tag.comment && ! tag.style)
       {
-          UdmDSTRReset(&tbuf);
-          UdmDSTRAppend(&tbuf, htok, last - htok);
-          Item.str = tbuf.data;
+        UdmDSTRReset(&tbuf);
+        UdmDSTRAppend(&tbuf, htok, last - htok);
+        Item.str = tbuf.data;
+        if (&Indexer->Conf->SectionHdrMatch == SectionMatch)
+        {
+          UdmVarListReplaceStr(&Doc->Sections, Item.section_name, Item.str);
+        }
+        else if (&Indexer->Conf->SectionGsrMatch == SectionMatch && doccs)
+        {
+          char *trmsrc= UdmTrim(Item.str, " \t\r\n");
+          size_t trmlen= strlen(trmsrc);
+          size_t cnvlen= trmlen * 12 + 1;
+          char *cnvsrc= (char*) UdmMalloc(cnvlen);
+          UdmVarListReplaceStr(&Doc->Sections, Item.section_name, Item.str);
+          if (trmsrc[0] && cnvsrc)
+          {
+            UDM_CONV lcs_dcs;
+            UdmConvInit(&lcs_dcs, Doc->lcs, doccs, UDM_RECODE_HTML);
+            UdmConv(&lcs_dcs, cnvsrc, cnvlen, trmsrc, trmlen + 1);
+            Item.str= cnvsrc;
+            Item.flags= UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION;
+            UdmTextListAdd(&Doc->TextList, &Item);
+            UdmFree(cnvsrc);
+          }
+        }
+        else
+        {
           UdmTextListAdd(&Doc->TextList, &Item);
+        }
       }
     }
   }
@@ -1039,6 +1115,72 @@ static int UdmParseSections (UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   return(UDM_OK);
 }
 
+
+
+/*
+  Create HTTP.Content variable if requested
+*/
+
+static int
+UdmVarListAddContentStr(UDM_VARLIST *Sections, const char *name,
+                        UDM_DOCUMENT *Doc)
+{
+  UDM_VAR *Sec;
+  if (Doc->Buf.content && (Sec= UdmVarListFind(Sections, name)))
+  {
+    size_t content_size=Doc->Buf.size - (Doc->Buf.content-Doc->Buf.buf);
+    if (content_size)
+    {
+      if (!(Sec->val= (char*) UdmRealloc(Sec->val, content_size + 1)))
+        return UDM_ERROR;
+      memcpy(Sec->val, Doc->Buf.content, content_size);
+      Sec->val[content_size]= '\0';
+      Sec->curlen= content_size;
+    }
+  }
+  return UDM_OK;
+}
+
+
+/*
+  Create "HTTP.LocalCharsetContent" variable if requested
+  Must be executed after character set guesser;
+*/
+static int
+UdmVarListAddLocalCharsetContentStr(const char *name,
+                                    UDM_DOCUMENT *Doc, UDM_CHARSET *doccs)
+{
+  UDM_VAR *Sec;
+  size_t srclen;
+  
+  if (Doc->Buf.content &&
+      (Sec= UdmVarListFind(&Doc->Sections, "HTTP.LocalCharsetContent")) &&
+      (srclen= Doc->Buf.size - (Doc->Buf.content-Doc->Buf.buf)))
+  {
+    const char *src= Doc->Buf.content;
+    if (!doccs)
+      doccs= UdmGetCharSet("iso-8859-1");
+    if (Doc->lcs == doccs)
+    {
+      Sec->val= (char*) UdmRealloc(Sec->val, srclen + 1);
+      memcpy(Sec->val, src, srclen);
+      Sec->val[srclen]= '\0';
+      Sec->curlen= srclen;
+    }
+    else
+    {
+      UDM_CONV dcs_lcs;
+      Sec->val= (char*) UdmRealloc(Sec->val, srclen * 8 + 1);
+      UdmConvInit(&dcs_lcs, doccs, Doc->lcs, UDM_RECODE_HTML_NONASCII);
+      srclen= UdmConv(&dcs_lcs, Sec->val, srclen * 8 + 1, src, srclen);
+      Sec->val[srclen]= '\0';
+      Sec->curlen= srclen;
+    }
+  }
+  return UDM_OK;
+}
+
+
 static int UdmDocParseContent(UDM_AGENT * Indexer, UDM_DOCUMENT * Doc){
   
 #ifdef USE_PARSER
@@ -1050,8 +1192,7 @@ static int UdmDocParseContent(UDM_AGENT * Indexer, UDM_DOCUMENT * Doc){
   const char  *ct=UdmVarListFindStr(&Doc->Sections,"Content-Type","");
   const char  *ce=UdmVarListFindStr(&Doc->Sections,"Content-Encoding","");
   int    result=UDM_OK;
-  UDM_VAR *Sec;
-  
+
   if(!strcmp(UDM_NULL2EMPTY(Doc->CurURL.filename), "robots.txt")) return UDM_OK;
   
 #ifdef HAVE_ZLIB
@@ -1110,20 +1251,6 @@ static int UdmDocParseContent(UDM_AGENT * Indexer, UDM_DOCUMENT * Doc){
   if(!real_content_type)real_content_type=ct;
   UdmVarListAddStr(&Doc->Sections,"Parser.Content-Type",real_content_type);
 
-  /* Create "Content" variable if requested */
-  if ((Sec= UdmVarListFind(&Doc->Sections, "HTTP.Content")))
-  {
-    size_t content_size=Doc->Buf.size - (Doc->Buf.content-Doc->Buf.buf);
-    if (content_size)
-    {
-      Sec->val= (char*) UdmRealloc(Sec->val, content_size + 1);
-      memcpy(Sec->val, Doc->Buf.content, content_size);
-      Sec->val[content_size]= '\0';
-      Sec->curlen= content_size;
-    }
-  }
-  
-  
 #ifdef HAVE_ZLIB
   if (Doc->method == UDM_METHOD_GET &&
       UDM_OK!=(result = UdmPutCachedCopy(Indexer, Doc)))
@@ -1157,9 +1284,13 @@ static int UdmDocParseContent(UDM_AGENT * Indexer, UDM_DOCUMENT * Doc){
   else if(!strncasecmp(real_content_type,"text/html",9))
   {
     UdmHTMLParse(Indexer,Doc);
-    UdmParseSections(Indexer, Doc);
+    UdmParseSections(Indexer, &Indexer->Conf->SectionMatch, Doc, NULL);
   }
-  else if(!strncasecmp(real_content_type, "text/xml",8))
+  else if(!strncasecmp(real_content_type, "text/xml",8) ||
+          !strncasecmp(real_content_type, "application/xml",15) ||
+          !strncasecmp(real_content_type, "application/rss+xml",19) ||
+          strstr(real_content_type, "xml") ||
+          strstr(real_content_type, "rss"))
   {
     int rc;
     UdmVarListReplaceStr(&Indexer->Conf->Vars, "XMLDefaultSection", "body");
@@ -1198,6 +1329,52 @@ static int UdmDocAddDocExtraHeaders(UDM_DOCUMENT *Doc){
   return UDM_OK;
 }
 
+static int
+UdmDocAddCookieHeaders(UDM_ENV *Conf, UDM_DOCUMENT *Doc)
+{
+  size_t    i;
+  UDM_DSTR cookie;
+  UdmDSTRInit(&cookie, 1024);
+  for (i= 0; i < Conf->Cookies.nvars; i++)
+  {
+    UDM_VAR *v= &Conf->Cookies.Var[i];
+    const char *domain, *path;
+    size_t short_dlen, long_dlen;
+    if (!(domain= strchr(v->name, '@')))
+      continue;
+    domain++;
+    if (!(path= strchr(domain, '/')))
+    {
+      continue;
+    }
+    else
+    {
+      size_t plen= strlen(path);
+      if (strncmp(Doc->CurURL.path, path, plen))
+        continue;
+    }
+    short_dlen= path-domain;
+    long_dlen= strlen(Doc->CurURL.hostname);
+    if (short_dlen > long_dlen)
+      continue;
+    if (strncasecmp(domain,
+                    Doc->CurURL.hostname + long_dlen - short_dlen, short_dlen))
+      continue;
+    if (cookie.size_data)
+      UdmDSTRAppend(&cookie, "; ", 2);
+    UdmDSTRAppend(&cookie, v->name, domain - v->name - 1);
+    UdmDSTRAppend(&cookie, "=", 1);
+    UdmDSTRAppendSTR(&cookie, v->val);
+  }
+  if (cookie.size_data)
+  {
+    UdmVarListReplaceStr(&Doc->RequestHeaders, "Cookie", cookie.data);
+  }
+  UdmDSTRFree(&cookie);
+  return UDM_OK;
+}
+
+
 static int UdmDocAddConfExtraHeaders(UDM_ENV *Conf,UDM_DOCUMENT *Doc){
   char    arg[128]="";
   const char  *lc;
@@ -1216,6 +1393,9 @@ static int UdmDocAddConfExtraHeaders(UDM_ENV *Conf,UDM_DOCUMENT *Doc){
       UdmVarListInsStr(&Doc->RequestHeaders,v->name+8,v->val);
   }
   
+  if (UdmVarListFindBool(&Conf->Vars, "UseCookie", 0))
+    UdmDocAddCookieHeaders(Conf, Doc);
+  
 #ifdef HAVE_ZLIB
   UdmVarListInsStr(&Doc->RequestHeaders,"Accept-Encoding","gzip,deflate,compress");
 #endif
@@ -1342,10 +1522,160 @@ static int UdmVarList2Spider(UDM_SPIDERPARAM *S,UDM_VARLIST *V){
   return UDM_OK;
 }
 
+
+static int
+UdmDocImportSections(UDM_AGENT *A, UDM_DOCUMENT *D, UDM_CHARSET *doccs)
+{
+  int rc= UDM_OK;
+  if (doccs && doccs != D->lcs)
+  {
+    UDM_DOCUMENT IDoc;
+    UdmDocInit(&IDoc);
+    UdmVarListReplaceLst(&IDoc.Sections, &D->Sections, NULL, "*");
+    if (UDM_OK != (rc= UdmURLAction(A, &IDoc, UDM_URL_ACTION_SQLIMPORTSEC)))
+      goto ex;
+    if (IDoc.TextList.nitems)
+    {
+      size_t i;
+      UDM_CONV conv;
+      /*
+         Convert imported sections from local
+         charset to document charset
+      */
+      UdmConvInit(&conv, D->lcs, doccs, UDM_RECODE_HTML);
+
+      for (i= 0; i < IDoc.TextList.nitems; i++)
+      {
+        UDM_TEXTITEM *I= &IDoc.TextList.Item[i];
+        size_t slen= strlen(I->str);
+        char *newval= (char*) UdmMalloc(slen * 12 + 1);
+        UdmConv(&conv, newval, slen * 12 + 1, I->str, slen + 1);
+        UDM_FREE(I->str);
+        I->str= newval;
+        I->flags= 0;
+        UdmTextListAdd(&D->TextList, I);
+      }
+    }
+    UdmDocFree(&IDoc);
+  }
+  else
+  {
+    if (UDM_OK != (rc= UdmURLAction(A, D, UDM_URL_ACTION_SQLIMPORTSEC)))
+      goto ex;
+  }
+ex:
+  return rc;
+}
+
+
+static int
+UdmParseHTTPResponseAndHeaders(UDM_AGENT *A, UDM_DOCUMENT *Doc)
+{
+  int status;
+  UdmParseHTTPResponse(A, Doc);
+  UdmDocProcessResponseHeaders(A, Doc);
+  UdmVarListAddContentStr(&Doc->Sections, "HTTP.Content", Doc);
+  UdmParseSections(A, &A->Conf->SectionHdrMatch, Doc, NULL);
+  UdmVarListLog(A, &Doc->Sections, UDM_LOG_DEBUG, "Response");
+  status= UdmVarListFindInt(&Doc->Sections,"Status",0);
+  UdmLog(A, UDM_LOG_EXTRA, "Status: %d %s", status, UdmHTTPErrMsg(status));
+  return status;
+}
+
+
+static int
+UdmDocGetRemote(UDM_AGENT *A, UDM_DOCUMENT *Doc,
+                const char *origurl, const char *aliasurl, const char *alias)
+{
+  int rc= UDM_OK, mp3type= UDM_MP3_UNKNOWN;
+  int state, start= Doc->method == UDM_METHOD_CHECKMP3 ||
+                    Doc->method == UDM_METHOD_CHECKMP3ONLY ? 1 : 0;
+  
+  if (Doc->method == UDM_METHOD_IMPORTONLY)
+  {
+    Doc->Buf.size= udm_snprintf(Doc->Buf.buf, Doc->Buf.maxsize,
+                               "HTTP/1.0 200 OK\r\n"
+                               "Content-Type: text/plain; charset=%s\r\n"
+                               "X-Method: ImportOnly\r\n\r\n"
+                               " ", Doc->lcs->name);
+    if(origurl != NULL)
+    {
+      UdmVarListReplaceStr(&Doc->Sections,"URL",origurl);
+      UdmVarListReplaceInt(&Doc->Sections, "URL_ID", UdmStrHash32(origurl));
+      UdmURLParse(&Doc->CurURL,origurl);
+    }
+    
+    UdmParseHTTPResponseAndHeaders(A, Doc);
+    return UDM_OK;
+  }
+  
+  for(state= start; state >= 0; state--)
+  {
+    int status;
+    const char *hdr= NULL;
+    
+    if(state==1)hdr="bytes=0-256";
+    if(mp3type==UDM_MP3_TAG)hdr="bytes=-128";
+    
+    UdmVarListReplaceInt(&Doc->Sections, "Status", UDM_HTTP_STATUS_UNKNOWN);
+    
+    if(aliasurl != NULL)
+    {
+      UdmVarListReplaceStr(&Doc->Sections,"URL",alias);
+      UdmVarListReplaceInt(&Doc->Sections, "URL_ID", UdmStrHash32(alias));
+      UdmURLParse(&Doc->CurURL,alias);
+    }
+    
+    UdmVarListLog(A, &Doc->RequestHeaders, UDM_LOG_DEBUG, "Request");
+      
+    if(hdr)
+    {
+      UdmVarListAddStr(&Doc->RequestHeaders, "Range", hdr);
+      UdmLog(A, UDM_LOG_INFO, "Range: [%s]", hdr);
+    }
+        
+    rc= UdmGetURL(A, Doc);
+      
+    if(hdr)
+    {
+      UdmVarListDel(&Doc->RequestHeaders, "Range");
+    }
+    
+    if(origurl != NULL)
+    {
+      UdmVarListReplaceStr(&Doc->Sections,"URL",origurl);
+      UdmVarListReplaceInt(&Doc->Sections, "URL_ID", UdmStrHash32(origurl));
+      UdmURLParse(&Doc->CurURL,origurl);
+    }
+    
+    if(rc != UDM_OK)
+      return rc;
+    
+    status= UdmParseHTTPResponseAndHeaders(A, Doc);
+    if (status !=206 && status != 200)
+      break;
+    
+    if(state == 1)  /* Needs guessing */
+    {
+      if(UDM_MP3_UNKNOWN != (mp3type= UdmMP3Type(Doc)))
+      {
+        UdmVarListReplaceStr(&Doc->Sections,"Content-Type","audio/mpeg");
+        if(Doc->method == UDM_METHOD_CHECKMP3ONLY &&
+           mp3type != UDM_MP3_TAG)
+          break;
+      }
+      if(Doc->method == UDM_METHOD_CHECKMP3ONLY)
+        break;
+    }
+  }
+  return rc;
+}
+
+
 __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
   int    result=UDM_OK;
   UDM_DOCUMENT  Doc;
-  const char  *url, *alias;
+  const char  *url, *alias, *logfmt;
   char    *origurl = NULL, *aliasurl = NULL;
   UDM_SERVER  *Server = NULL;
   char    reason[1024] = "";
@@ -1489,7 +1819,10 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
     return result;
   }
   
-  if(Doc.method!=UDM_METHOD_DISALLOW && Doc.method!=UDM_METHOD_VISITLATER){
+  if(Doc.method != UDM_METHOD_DISALLOW &&
+     Doc.method != UDM_METHOD_VISITLATER &&
+     Doc.method != UDM_METHOD_IMPORTONLY)
+  {
     const char *UseRobotsTxtURL = UdmVarListFindStr(&Server->Vars, "UseRobotsTxtURL", NULL);
     UdmDocAddDocExtraHeaders(&Doc);
     if(!strncasecmp(UDM_NULL2EMPTY(Doc.CurURL.schema), "http", 4) || UseRobotsTxtURL) {
@@ -1605,9 +1938,9 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
     return result;
   }
 
-  if(Doc.method!=UDM_METHOD_DISALLOW && Doc.method!=UDM_METHOD_VISITLATER){
-    int  start,state,status=0;
-    int  mp3type=UDM_MP3_UNKNOWN;
+  if(Doc.method!=UDM_METHOD_DISALLOW && Doc.method!=UDM_METHOD_VISITLATER)
+  {
+    int  status=0;
     
     if(!(Indexer->flags&UDM_FLAG_REINDEX)){
       const char *l=UdmVarListFindStr(&Doc.Sections,"Last-Modified",NULL);
@@ -1616,70 +1949,12 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
     
     UDM_THREADINFO(Indexer,"Getting",url);
     
-    start=(Doc.method==UDM_METHOD_CHECKMP3 || Doc.method==UDM_METHOD_CHECKMP3ONLY)?1:0;
+    if (UDM_OK != (result= UdmDocGetRemote(Indexer, &Doc,
+                                           origurl, aliasurl, alias)))
+      goto ex;
     
-    for(state=start;state>=0;state--){
-      const char  *hdr=NULL;
-      
-      if(state==1)hdr="bytes=0-256";
-      if(mp3type==UDM_MP3_TAG)hdr="bytes=-128";
-      
-      UdmVarListReplaceInt(&Doc.Sections, "Status", UDM_HTTP_STATUS_UNKNOWN);
-      
-      if(aliasurl != NULL) {
-        UdmVarListReplaceStr(&Doc.Sections,"URL",alias);
-        UdmVarListReplaceInt(&Doc.Sections, "URL_ID", UdmStrHash32(alias));
-        UdmURLParse(&Doc.CurURL,alias);
-      }
-        
-      UdmVarListLog(Indexer,&Doc.RequestHeaders,UDM_LOG_DEBUG,"Request");
-        
-      if(hdr) {
-              UdmVarListAddStr(&Doc.RequestHeaders, "Range", hdr);
-        UdmLog(Indexer, UDM_LOG_INFO, "Range: [%s]", hdr);
-      }
-        
-      result = UdmGetURL(Indexer, &Doc);
-        
-      if(hdr) {
-              UdmVarListDel(&Doc.RequestHeaders, "Range");
-      }
-        
-      if(origurl != NULL) {
-        UdmVarListReplaceStr(&Doc.Sections,"URL",origurl);
-        UdmVarListReplaceInt(&Doc.Sections, "URL_ID", UdmStrHash32(origurl));
-        UdmURLParse(&Doc.CurURL,origurl);
-      }
-      
-      if(result!=UDM_OK)
-      {
-        UDM_FREE(origurl);
-        UDM_FREE(aliasurl);
-        UdmDocFree(&Doc);
-        UdmViolationExit(paran);
-        return result;
-      }
-      
-      UdmParseHTTPResponse(Indexer,&Doc);
-      UdmDocProcessResponseHeaders(Indexer,&Doc);
-      UdmVarListLog(Indexer,&Doc.Sections,UDM_LOG_DEBUG,"Response");
-      
-      status=UdmVarListFindInt(&Doc.Sections,"Status",0);
-      
-      UdmLog(Indexer, UDM_LOG_EXTRA, "Status: %d %s", status, UdmHTTPErrMsg(status));
+    status= UdmVarListFindInt(&Doc.Sections,"Status",0);
 
-      if(status!=206 && status!=200)
-        break;
-      
-      if(state==1){  /* Needs guessing */
-        if(UDM_MP3_UNKNOWN != (mp3type=UdmMP3Type(&Doc))){
-          UdmVarListReplaceStr(&Doc.Sections,"Content-Type","audio/mpeg");
-          if(Doc.method == UDM_METHOD_CHECKMP3ONLY && mp3type != UDM_MP3_TAG) break;
-        }
-        if(Doc.method == UDM_METHOD_CHECKMP3ONLY) break;
-      }
-    }
-    
     /* Add URL from Location: header */
     /* This is to give a chance for  */
     /* a concurent thread to take it */
@@ -1704,9 +1979,11 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
     }
     
     if(status==UDM_HTTP_STATUS_OK || status==UDM_HTTP_STATUS_PARTIAL_OK){
-         size_t    wordnum;
-      const char *lang = NULL;
-         
+      size_t    wordnum;
+      const char *lang= NULL;
+      const char *csname;
+      UDM_CHARSET *doccs;
+      
       UDM_THREADINFO(Indexer,"Parsing",url);
       
       result = UdmDocParseContent(Indexer, &Doc);
@@ -1737,11 +2014,46 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
       UdmGuessCharSet(Indexer, &Doc, &Indexer->Conf->LangMaps, Indexer->LangMap);
         
       UdmLog(Indexer, UDM_LOG_EXTRA, "Guesser: Lang: %s, Charset: %s",
-             lang = UdmVarListFindStr(&Doc.Sections,"Content-Language",""),
-             UdmVarListFindStr(&Doc.Sections,"Charset",""));
+             lang= UdmVarListFindStr(&Doc.Sections,"Content-Language",""),
+             csname= UdmVarListFindStr(&Doc.Sections,"Charset",""));
       
-      UdmParseURLText(Indexer,&Doc);
-      UdmParseHeaders(Indexer,&Doc);
+      /*
+        Convert Href variables into LocalCharset.
+        Typically, for RSS indexing.
+      */
+      doccs= UdmGetCharSet(csname);
+      if (doccs && doccs != Doc.lcs)
+      {
+        size_t h;
+        UDM_CONV conv;
+        UdmConvInit(&conv, doccs, Doc.lcs, UDM_RECODE_HTML);
+        for (h= 0; h < Doc.Hrefs.nhrefs; h++)
+        {
+          UdmVarListConvert(&Doc.Hrefs.Href[h].Vars, &conv);
+        }
+      }
+      if (Doc.Spider.index)
+      {
+        /*
+          Doc.Spider.index can be unset e.g. in HTML parser by:
+          <META NAME="ROBOTS" CONTENT="NOINDEX">
+          Don't parse URL and headers in this case.
+        */
+        UdmParseURLText(Indexer,&Doc);
+        UdmParseHeaders(Indexer,&Doc);
+      }
+      
+      if (UDM_OK != (result= UdmDocImportSections(Indexer, &Doc, doccs)))
+        goto ex;
+
+      /* Create "HTTP.LocalCharsetContent" variable if requested */
+      if (UDM_OK!=  (result= UdmVarListAddLocalCharsetContentStr("HTTP.LocalCharsetContent", &Doc, doccs)))
+        goto ex;
+      
+      /* Process "afterguesser" user-defined sections */
+      UDM_GETLOCK(Indexer,UDM_LOCK_CONF);
+      UdmParseSections(Indexer, &Indexer->Conf->SectionGsrMatch, &Doc, doccs);
+      UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
 
       if (Doc.method != UDM_METHOD_HREFONLY)
       {
@@ -1775,6 +2087,8 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
             UdmVarListDel(&Doc.Sections, "CachedCopy");
             UdmWordListFree(&Doc.Words);
             UdmCrossListFree(&Doc.CrossWords);
+            if (sec_method == UDM_METHOD_VISITLATER)
+              UdmHrefListFree(&Doc.Hrefs);
           }
           UdmVarListFree(&TmpList);
           UdmLog(Indexer,UDM_LOG_DEBUG,"SectionFilter: %s",reason);
@@ -1822,6 +2136,30 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
     if(UDM_OK!=(result=UdmStoreHrefs(Indexer)))
       result=UDM_ERROR;
   }
+
+ex:
+
+  /* Lock mutex: localtime and printf are not reentrant */
+  UDM_GETLOCK(Indexer,UDM_LOCK_CONF);
+  if ((logfmt= UdmVarListFindStr(&Indexer->Conf->Vars, "CustomLog", NULL)))
+  {
+    UDM_DSTR line;
+    char buf[256];
+    const char *datefmt;
+    struct tm *tim;
+    time_t t;
+    UdmDSTRInit(&line, 1024);
+    t= time(NULL);
+    tim= localtime(&t);
+    datefmt= UdmVarListFindStr(&Indexer->Conf->Vars, "DateFormat", "%a, %d %b %Y, %X");
+    strftime(buf, sizeof(buf), datefmt , tim);
+    UdmVarListReplaceStr(&Doc.Sections, "CurrentTime", buf);
+    UdmVarListReplaceInt(&Doc.Sections, "PID", (int) getpid());
+    UdmDSTRParse(&line, logfmt, &Doc.Sections);
+    fprintf(stdout, "%s\n", line.data);
+    UdmDSTRFree(&line);
+  }
+  UDM_RELEASELOCK(Indexer,UDM_LOCK_CONF);
   
   if(result!=UDM_OK)
   {
