@@ -85,7 +85,6 @@ UDM_AGENT **ThreadIndexers;
 #ifdef WIN32
 HANDLE *threads;
 #ifdef CONSOLE
-#define getpid(x) _getpid(x)
 #define localtime_r(x, y) localtime(x)
 #define strptime(x, y, z) (0)
 #endif
@@ -647,18 +646,22 @@ struct udm_indcmd2str_st
 
 static struct udm_indcmd2str_st indcmd[]=
 {
-  {"index",      UDM_IND_INDEX},
-  {"statistics", UDM_IND_STAT},
-  {"create",     UDM_IND_CREATE},
-  {"drop",       UDM_IND_DROP},
-  {"delete",     UDM_IND_DELETE},
-  {"referers",   UDM_IND_REFERERS},
-  {"sqlmon",     UDM_IND_SQLMON},
-  {"checkconf" , UDM_IND_CHECKCONF},
-  {"blob",       UDM_IND_MULTI2BLOB},
-  {"export",     UDM_IND_EXPORT},
-  {"wordstat",   UDM_IND_WRDSTAT},
-  {"",           UDM_IND_INDEX}
+  {"index",         UDM_IND_INDEX},
+  {"statistics",    UDM_IND_STAT},
+  {"create",        UDM_IND_CREATE},
+  {"drop",          UDM_IND_DROP},
+  {"delete",        UDM_IND_DELETE},
+  {"referers",      UDM_IND_REFERERS},
+  {"sqlmon",        UDM_IND_SQLMON},
+  {"checkconf",     UDM_IND_CHECKCONF},
+  {"blob",          UDM_IND_MULTI2BLOB},
+  {"export",        UDM_IND_EXPORT},
+  {"wordstat",      UDM_IND_WRDSTAT},
+  {"rewriteurl",    UDM_IND_REWRITEURL},
+  {"rewritelimits", UDM_IND_REWRITELIMITS},
+  {"hashspell",     UDM_IND_HASHSPELL},
+  {"dumpspell",     UDM_IND_DUMPSPELL},
+  {"",              UDM_IND_INDEX}
 };
 
 static enum udm_indcmd UdmIndCmd(const char *cmdname)
@@ -1349,7 +1352,8 @@ int main(int argc, char **argv)
   {
     add_servers=0;
     load_langmaps=0;
-    load_spells=0;
+    if (cmd != UDM_IND_HASHSPELL && cmd  != UDM_IND_DUMPSPELL)
+      load_spells=0;
   }
      
   flags|=add_servers;
@@ -1416,7 +1420,37 @@ int main(int argc, char **argv)
     UdmBitrixIndex(&Main);
     exit(0);
   }
-
+  
+  if (cmd == UDM_IND_HASHSPELL)
+  {
+    char errmsg[256];
+    int rc; 
+    if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells, errmsg, sizeof(errmsg)))||
+        UDM_OK != (rc= UdmSpellListListWriteHash(&Conf.Spells, errmsg, sizeof(errmsg))))
+    {
+      fprintf(stderr, "error: %s\n", errmsg);
+      exit(1);
+    }
+    exit(0);
+  }
+  
+  if (cmd == UDM_IND_DUMPSPELL)
+  {
+    char errmsg[256];
+    int rc, spflags= UDM_SPELL_NOPREFIX;
+    if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells,
+                                            errmsg, sizeof(errmsg)))||
+        UDM_OK != (rc= UdmAffixListListLoad(&Conf.Affixes, spflags,
+                                            errmsg, sizeof(errmsg)))||
+        UDM_OK != (rc= UdmSpellDump(&Conf.Spells, &Conf.Affixes,
+                                    errmsg, sizeof(errmsg))))
+    {
+      fprintf(stderr, "error: %s\n", errmsg);
+      exit(1);
+    }
+    exit(0);
+  }
+  
   if (cmd==UDM_IND_SQLMON)
   {
     UDM_SQLMON_PARAM prm;
@@ -1446,6 +1480,16 @@ int main(int argc, char **argv)
     UdmURLAction(&Main, NULL, UDM_URL_ACTION_WRDSTAT);
     exit(0);
   }
+  else if (cmd == UDM_IND_REWRITEURL)
+  {
+    UdmRewriteURL(&Main);
+    exit(0);
+  }     
+  else if (cmd == UDM_IND_REWRITELIMITS)
+  {
+    UdmRewriteLimits(&Main);
+    exit(0);
+  }
      
   if (url_filename && strcmp(url_filename,"-"))
   {
@@ -1541,8 +1585,8 @@ int main(int argc, char **argv)
         if (block)
         {
           /* Check that another instance isn't running and create PID file. */
-          
-          sprintf(pidname,"%s/%s",UDM_VAR_DIR,"indexer.pid");
+          const char *vardir= UdmVarListFindStr(&Conf.Vars,"VarDir",UDM_VAR_DIR);
+          sprintf(pidname,"%s/%s", vardir ,"indexer.pid");
           pid_fd = open(pidname,O_CREAT|O_EXCL|O_WRONLY,0644);
           if (pid_fd < 0)
           {
