@@ -83,7 +83,7 @@ static void UdmPrintLangMap(UDM_LANGMAP * map){
 	printf("Charset:  %s\n",map->charset);
 	printf("\n");
 	printf("\n");
-	UdmSort(map->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (qsort_cmp)UdmLMcmpCount);
+	UdmSort(map->memb, UDM_LM_HASHMASK + 1, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpCount);
 	for(i = 0; i < UDM_LM_TOPCNT; i++) {
 		char * s;
 		if(!map->memb[i].count)break;
