@@ -12,6 +12,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include "udm_common.h"
 #include "udm_parsedate.h"
 #include "udm_xmalloc.h"
 #include "udm_utils.h"
@@ -101,7 +102,7 @@ char * UdmDateParse(const char * datestring)
 			offset = (int) ( (SDPALIGN)pos - (SDPALIGN)datestring);	
 			offset += 2; /* skip comma and whitespace */
 		}
-		copy = malloc(strlen(datestring+offset)+1);
+		copy = UdmMalloc(strlen(datestring+offset)+1);
 		strcpy(copy,datestring+offset);
 		copyref = copy; /* save the pointer for deallocation later */
 	
@@ -158,13 +159,13 @@ char * UdmDateParse(const char * datestring)
 
 		/* we allocate 4 extra characters for separators and \0  */
 		allocsize = strlen(date)+strlen(month)+strlen(year)+strlen(timebuf)+ 4;
-		res = malloc(allocsize);
+		res = UdmMalloc(allocsize);
 		udm_snprintf(res,allocsize,"%s-%02i-%02i %s",year,get_month(month),atoi(date),timebuf);
 		res[allocsize-1] = '\0';
 		/* clean up nicely. we dont want memory leaks within 100 lines...*/
 		UDM_FREE(copyref);
 	} else {
-		if ((res = malloc(20)))
+		if ((res = UdmMalloc(20)))
 			sprintf(res, "1970-01-01 00:01");
 	}
 	return res;
