@@ -23,10 +23,10 @@
 #include <sys/types.h>
 
 #include "udm_common.h"
+#include "udm_utils.h"
 #include "udm_uniconv.h"
 #include "udm_vars.h"
 #include "udm_textlist.h"
-#include "udm_utils.h"
 #include "udm_parsexml.h"
 #include "udm_hrefs.h"
 #include "udm_sgml.h"
@@ -35,6 +35,7 @@ typedef struct
 {
   UDM_AGENT *Indexer;
   UDM_DOCUMENT *Doc;
+  UDM_HREF Href;
   int body_sec;
   const char *XMLDefaultSection;
   char *sec;
@@ -55,6 +56,7 @@ typedef struct
 #define UDM_XML_TEXT		'T'
 #define UDM_XML_QUESTION	'?'
 #define UDM_XML_EXCLAM		'!'
+#define UDM_XML_CDATA		'D'
 
 #define UDM_XML_OK	0
 #define UDM_XML_ERROR	1
@@ -74,6 +76,7 @@ typedef struct xml_stack_st
   const char *beg;
   const char *cur;
   const char *end;
+  char question;
   void *user_data;
   int (*enter)(struct xml_stack_st *st, const char *val, size_t len);
   int (*value)(struct xml_stack_st *st, const char *val, size_t len);
@@ -87,6 +90,7 @@ static const char *UdmLex2str (int lex)
     case UDM_XML_EOF:      return("EOF");
     case UDM_XML_STRING:   return("STRING");
     case UDM_XML_IDENT:    return("IDENT");
+    case UDM_XML_CDATA:    return("CDATA");
     case UDM_XML_EQ:       return("'='");
     case UDM_XML_LT:       return("'<'");
     case UDM_XML_GT:       return("'>'");
@@ -111,16 +115,17 @@ static int UdmXMLScan (UDM_XML_PARSER *p, UDM_XML_ATTR *a)
 
   for(; (p->cur < p->end) && strchr(" \t\r\n", p->cur[0]); p->cur++);
 
-  if (p->cur >= p->end) {
+  if (p->cur >= p->end)
+  {
     a->beg = p->end;
     a->end = p->end;
-          lex = UDM_XML_EOF;
+    lex = UDM_XML_EOF;
     goto ret;
   }
 
   a->beg = p->cur;
   a->end = p->cur;
-
+  
   if (! memcmp(p->cur, "<!--", 4))
   {
     for(; (p->cur < p->end) && memcmp(p->cur, "-->", 3); p->cur++);
@@ -129,6 +134,20 @@ static int UdmXMLScan (UDM_XML_PARSER *p, UDM_XML_ATTR *a)
     a->end = p->cur;
     lex = UDM_XML_COMMENT;
   }
+  else if (!memcmp(p->cur, "<![CDATA[",9))
+  {
+    p->cur+= 9;
+    for (; p->cur < p->end ; p->cur++)
+    {
+      if (p->cur[0] == ']' && p->cur[1] == ']' && p->cur[2] == '>')
+      {
+        p->cur+= 3;
+        a->end= p->cur;
+        break;
+      }
+    }
+    lex= UDM_XML_CDATA;
+  }
   else if (strchr("?=/<>!", p->cur[0]))
   {
     p->cur++;
@@ -163,11 +182,17 @@ ret:
 
 static int UdmXMLValue (UDM_XML_PARSER *st, const char *str, size_t len)
 {
+#ifdef DEBUG_XML
+  fprintf(stderr, "UdmXMLValue: %.*s\n", len, str);
+#endif
   return((st->value) ? (st->value)(st, str, len) : UDM_XML_OK);
 }
 
 static int UdmXMLEnter(UDM_XML_PARSER *st, const char *str, size_t len)
 {
+#ifdef DEBUG_XML
+  fprintf(stderr, "UdmXMLEnter: %.*s\n", len, str);
+#endif
   if ((st->attrend - st->attr + len + 1) > sizeof(st->attr))
   {
     sprintf(st->errstr, "To deep XML");
@@ -199,6 +224,9 @@ static int UdmXMLLeave(UDM_XML_PARSER *p, const char *str, size_t slen)
   char g[32];
   int rc;
 
+#ifdef DEBUG_XML
+  fprintf(stderr, "UdmXMLLeave: %.*s\n", slen, str);
+#endif
   /* Find previous '.' or beginning */
   for(e = p->attrend; (e > p->attr) && (e[0] != '.'); e--);
   glen = (e[0] == '.') ? (p->attrend - e - 1) : p->attrend - e;
@@ -233,14 +261,22 @@ int UdmXMLParser (UDM_XML_PARSER *p, const char *str, size_t len)
     if (p->cur[0] == '<')
     {
       int lex;
-      int question = 0;
       int exclam = 0;
+      p->question= 0;
 
       lex = UdmXMLScan(p, &a);
 
       if (UDM_XML_COMMENT==lex)
         continue;
 
+      if (UDM_XML_CDATA == lex)
+      {
+        a.beg+= 9;
+        a.end-= 3;
+        UdmXMLValue(p, a.beg, a.end - a.beg);
+        continue;
+      }
+      
       lex = UdmXMLScan(p, &a);
 
       if (UDM_XML_SLASH == lex)
@@ -264,7 +300,7 @@ int UdmXMLParser (UDM_XML_PARSER *p, const char *str, size_t len)
       else if (UDM_XML_QUESTION == lex)
       {
         lex = UdmXMLScan(p, &a);
-        question = 1;
+        p->question = 1;
       }
 
       if (UDM_XML_IDENT == lex)
@@ -323,7 +359,7 @@ int UdmXMLParser (UDM_XML_PARSER *p, const char *str, size_t len)
       }
 gt:
 
-      if (question)
+      if (p->question)
       {
         if (lex != UDM_XML_QUESTION)
         {
@@ -433,11 +469,25 @@ size_t UdmXMLErrorLineno (UDM_XML_PARSER *p)
 static int startElement(UDM_XML_PARSER *parser, const char *name, size_t l)
 {
   XML_PARSER_DATA *D = parser->user_data;
-
+  UDM_VARLIST *Vars= &D->Indexer->Conf->XMLEnterHooks;
+  const char *val;
+  
   UDM_FREE(D->sec);
   D->sec = (char*)UdmStrndup(name, l);
   UDM_FREE(D->secpath);
   D->secpath = UdmStrndup(name, l);
+  if (Vars->nvars && (val= UdmVarListFindStr(Vars, D->secpath, NULL)))
+  {
+    if (!strcasecmp(val, "HrefVarInit"))
+    {
+      UdmVarListFree(&D->Href.Vars);
+    }
+    else if (!strcasecmp(val, "HrefInit"))
+    {
+      UdmHrefFree(&D->Href);
+      UdmHrefInit(&D->Href);
+    }
+  }
   return(UDM_XML_OK);
 }
 
@@ -446,8 +496,25 @@ static int endElement(UDM_XML_PARSER *parser, const char *name, size_t l)
   XML_PARSER_DATA *D = parser->user_data;
   size_t i = l;
 
+  if (D->Indexer->Conf->XMLLeaveHooks.nvars)
+  {
+    char *prevname= UdmStrndup(name, l);
+    const char *val;
+    if ((val= UdmVarListFindStr(&D->Indexer->Conf->XMLLeaveHooks,
+                                prevname, NULL)))
+    {
+      UDM_DOCUMENT *Doc= D->Doc;
+      D->Href.referrer = UdmVarListFindInt(&Doc->Sections, "Referrer-ID", 0);
+      D->Href.hops = 1 + UdmVarListFindInt(&Doc->Sections, "Hops", 0);
+      D->Href.site_id = UdmVarListFindInt(&Doc->Sections, "Site_id", 0);
+      D->Href.method = UDM_METHOD_GET;
+      UdmHrefListAdd(&Doc->Hrefs, &D->Href);
+    }
+    UdmFree(prevname);
+  }
+
   while (--i && name[i] != '.');
-  
+
   UDM_FREE(D->sec);
   D->sec = (char *)UdmStrndup(name, i);
   UDM_FREE(D->secpath);
@@ -460,11 +527,31 @@ static int Text (UDM_XML_PARSER *parser, const char *s, size_t len)
   XML_PARSER_DATA *D= parser->user_data;
   UDM_DOCUMENT *Doc= D->Doc;
   UDM_TEXTITEM  Item;
+  UDM_VARLIST  *Vars= &D->Indexer->Conf->XMLDataHooks;
   UDM_VAR    *Sec;
+  const char *val;
   size_t slen;
 
   if (D->sec == NULL) return(UDM_XML_OK);
 
+  /* Process XMLDataHook commands */
+  if ((val= UdmVarListFindStr(Vars, D->sec, NULL)))
+  {
+    if (!strcasecmp(val, "HrefSet"))
+    {
+      UDM_FREE(D->Href.url);
+      D->Href.url= UdmStrndup(s, (size_t)len);
+      UdmSGMLUnescape(D->Href.url);
+    }
+    else if (!strcasecmp(val, "HrefVarAdd"))
+    {
+      char *str= UdmStrndup(s, len);
+      UdmVarListReplaceStr(&D->Href.Vars, D->sec, str);
+      UdmFree(str);
+      
+    }
+  }
+  
   bzero((void*)&Item, sizeof(Item));
   Item.str= UdmStrndup(s, (size_t)len);
   if((Sec = UdmVarListFind(&Doc->Sections, D->sec)))
@@ -489,18 +576,33 @@ static int Text (UDM_XML_PARSER *parser, const char *s, size_t len)
       (slen = strlen(D->secpath)) >= 4 &&
       !strncasecmp(&D->secpath[slen - 5], ".href", 5))
   {
-    UDM_HREF  Href;
-    char *u = UdmStrndup(s, (size_t)len);
-    UdmSGMLUnescape(u);
-    UdmHrefInit(&Href);
-    Href.referrer = UdmVarListFindInt(&Doc->Sections, "Referrer-ID", 0);
-    Href.hops = 1 + UdmVarListFindInt(&Doc->Sections, "Hops", 0);
-    Href.site_id = UdmVarListFindInt(&Doc->Sections, "Site_id", 0);
-    Href.url = u;
-    Href.method = UDM_METHOD_GET;
-    UdmHrefListAdd(&Doc->Hrefs, &Href);
+    UdmHrefFree(&D->Href);
+    UdmHrefInit(&D->Href);
+    D->Href.url= UdmStrndup(s, (size_t)len);
+    UdmSGMLUnescape(D->Href.url);
+    D->Href.referrer = UdmVarListFindInt(&Doc->Sections, "Referrer-ID", 0);
+    D->Href.hops = 1 + UdmVarListFindInt(&Doc->Sections, "Hops", 0);
+    D->Href.site_id = UdmVarListFindInt(&Doc->Sections, "Site_id", 0);
+    D->Href.method = UDM_METHOD_GET;
+    UdmHrefListAdd(&Doc->Hrefs, &D->Href);
   }
 
+
+  if ((slen == 12 && !strcasecmp(D->secpath, "rss.encoding")) ||
+      (parser->question && slen == 12 &&
+       !strcasecmp(D->secpath, "xml.encoding")))
+  {
+    char buf[64];
+    if (len > 0 && len < sizeof(buf))
+    {
+      const char *csname;
+      memcpy(buf, s, len);
+      buf[len]= '\0';
+      csname= UdmCharsetCanonicalName(buf);
+      if (csname)
+        UdmVarListReplaceStr(&Doc->Sections, "Meta-Charset", csname);
+    }
+  }
   return(UDM_XML_OK);
 }
 
@@ -575,6 +677,8 @@ int UdmXMLParse(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   }
 
   UdmXMLParserFree(&parser);
+  UDM_FREE(Data.sec);
   UDM_FREE(Data.secpath);
+  UdmHrefFree(&Data.Href);
   return res;
 }
