@@ -202,70 +202,83 @@ static int open_host(UDM_AGENT *Agent,UDM_DOCUMENT *Doc)
 
 static int UdmHTTPGet(UDM_AGENT *Agent,UDM_DOCUMENT *Doc)
 {
-    int fd,res=0;
-    fd_set sfds;
-    time_t start_time;
-    int status;
-    size_t buf_size=UDM_NET_BUF_SIZE;
-    struct timeval tv;
+  int fd,res=0;
+  fd_set sfds;
+  time_t start_time;
+  int status;
+  size_t buf_size=UDM_NET_BUF_SIZE;
+  struct timeval tv;
     
-    Doc->Buf.size=0;
+  Doc->Buf.size=0;
 
-    /* Connect to HTTP or PROXY server */
-    if( (fd=open_host(Agent,Doc)) < 0 )
-       return(fd);
+  /* Connect to HTTP or PROXY server */
+  if ((fd= open_host(Agent,Doc)) < 0)
+   return fd;
 
-    /* Send HTTP request */
-    if( UdmSend(fd, Doc->Buf.buf, strlen(Doc->Buf.buf), 0) < 0 )
-       return -1;
+  /* Send HTTP request */
+  if(UdmSend(fd, Doc->Buf.buf, strlen(Doc->Buf.buf), 0) < 0 )
+    return -1;
 
-    /* Retrieve response */
-    tv.tv_sec = (long) Doc->Spider.read_timeout;
-    tv.tv_usec = 0;
+  /* Retrieve response */
+  tv.tv_sec= (long) Doc->Spider.read_timeout;
+  tv.tv_usec= 0;
 
-    start_time=time(NULL);
-    while( 1 ){
-       FD_ZERO( &sfds );
-       FD_SET( fd, &sfds );
+  start_time= time(NULL);
+  while (1)
+  {
+    FD_ZERO(&sfds);
+    FD_SET(fd, &sfds);
 
-       status=select(FD_SETSIZE, &sfds, NULL, NULL, &tv);
-       if( status == -1 ){
-               res=UDM_NET_ERROR;
-               break;
-       }else
-       if( status == 0 ){
-               res=UDM_NET_TIMEOUT;
-               break;
-       }else{
-           if(FD_ISSET(fd,&sfds))
-           {
-		if(Doc->Buf.size+buf_size > Doc->Buf.maxsize)
-			buf_size = Doc->Buf.maxsize - Doc->Buf.size;
-		else
-			buf_size = UDM_NET_BUF_SIZE;
-		
-               status = recv(fd, Doc->Buf.buf+Doc->Buf.size, buf_size, 0);
-               if( status < 0 ){
-                   res = status;
-                   break;
-               }else 
-               if( status == 0 ){
-                   if(Doc->Spider.doc_timeout < (time(NULL) - start_time)){
-                   	res = UDM_NET_TIMEOUT;
-		   }
-		   break;
-               }else{
-                   Doc->Buf.size += status;
-		   start_time = time(NULL);
-                   if( Doc->Buf.size == Doc->Buf.maxsize )
-                       break;
-               }
-           }else
-               break;
-       }
+    status= select(FD_SETSIZE, &sfds, NULL, NULL, &tv);
+    if (status == -1)
+    {
+      res= UDM_NET_ERROR;
+      break;
     }
-    closesocket(fd);
-    return(res);
+    else if (status == 0)
+    {
+      res= UDM_NET_TIMEOUT;
+      break;
+    }
+    else
+    {
+      if (FD_ISSET(fd,&sfds))
+      {
+        if(Doc->Buf.size + buf_size > Doc->Buf.maxsize)
+          buf_size= Doc->Buf.maxsize - Doc->Buf.size;
+        else
+          buf_size= UDM_NET_BUF_SIZE;
+
+        status= recv(fd, Doc->Buf.buf+Doc->Buf.size, buf_size, 0);
+        if (status < 0)
+        {
+          res= status;
+          break;
+        }
+        else if (status == 0)
+        {
+          if(Doc->Spider.doc_timeout < (time(NULL) - start_time))
+          {
+            res = UDM_NET_TIMEOUT;
+          }
+          break;
+        }
+        else
+        {
+          Doc->Buf.size += status;
+          start_time = time(NULL);
+          if(Doc->Buf.size == Doc->Buf.maxsize)
+            break;
+        }
+      }
+      else
+      {
+        break;
+      }
+    }
+  }
+  closesocket(fd);
+  return res;
 }
 #endif
 
@@ -1216,7 +1229,7 @@ static char **loaddir (const char *path, int sort) {
 
   closedir(dir);
 
-  if (sort) UdmSort(_, nrecs, sizeof(char *), (qsort_cmp)cmpdirent);
+  if (sort) UdmSort(_, nrecs, sizeof(char *), (udm_qsort_cmp)cmpdirent);
   _[nrecs] = NULL;
   return(_);
 }
@@ -1311,7 +1324,12 @@ static int UdmFILEGet(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
 			case EACCES: status=403;break; /* Forbidden*/
 			default: status=500;
 		}
-		sprintf(Doc->Buf.buf,"HTTP/1.0 %d %s\r\nX-Reason: %s\r\n\r\n",status,UdmHTTPErrMsg(status),strerror(errno));
+		sprintf(Doc->Buf.buf,"HTTP/1.0 %d %s\r\n"
+		                     "X-Reason: %d:%s\r\n"
+		                     "X-Statname: '%s'\r\n"
+				     "\r\n",
+		        status,UdmHTTPErrMsg(status),errno,
+		        strerror(errno), mystatname);
 		Doc->Buf.size=strlen(Doc->Buf.buf);
 		return 0;
 	}
@@ -1395,7 +1413,11 @@ static int UdmFILEGet(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc){
 				case EACCES: status=403;break; /* Forbidden*/
 				default: status=500;
 			}
-			sprintf(Doc->Buf.buf,"HTTP/1.0 %d %s\r\n\r\n",status,UdmHTTPErrMsg(status));
+			sprintf(Doc->Buf.buf,
+			        "HTTP/1.0 %d %s\r\n"
+			        "X-Reason: Loaddir returned no results for '%s'\r\n"
+			        "\r\n",
+			        status, UdmHTTPErrMsg(status), openname);
 			Doc->Buf.size=strlen(Doc->Buf.buf);
 			return 0;
 		}
@@ -1479,117 +1501,137 @@ static int UdmBuildHTTPRequest(UDM_DOCUMENT *Doc){
 }
 
 
+static void
+UdmCreateErrorResponse(UDM_DOCUMENT *Doc, int code)
+{
+  Doc->Buf.size= udm_snprintf(Doc->Buf.buf, Doc->Buf.maxsize,
+                              "HTTP/1.0 %d %s\r\n\r\n",
+                              code, UdmHTTPErrMsg(code));
+  UdmVarListReplaceInt(&Doc->Sections, "Status", code);
+}
 
-int UdmGetURL(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
-	int		res=0;
-	const char	*proxy;
-	int             mirror_period, found_in_mirror = 0;
 
-	UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
-	proxy=UdmVarListFindStr(&Doc->RequestHeaders,"Proxy",NULL);
-	mirror_period = UdmVarListFindInt(&Doc->Sections, "MirrorPeriod", -1);
-	UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+int UdmGetURL(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc)
+{
+  const char  *proxy;
+  int mirror_period, found_in_mirror= 0, res= 0;
 
-	UdmBuildHTTPRequest(Doc);
-	
-	/* If mirroring is enabled */
-	if (mirror_period >= 0) {
-		/* on u_m==0 it returned by mtime from mirrorget */
-		/* but we knew that it should be put in mirror  */
-		
-		res = UdmMirrorGET(Indexer, Doc, &Doc->CurURL);
-		
-		if(!res){
-			UdmLog(Indexer, UDM_LOG_DEBUG, "Taken from mirror");
-			found_in_mirror = 1;
-		}
-	}
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  proxy= UdmVarListFindStr(&Doc->RequestHeaders,"Proxy",NULL);
+  mirror_period = UdmVarListFindInt(&Doc->Sections, "MirrorPeriod", -1);
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
 
-	if (!found_in_mirror) {
-	if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "exec")) {
-		res = UdmExecGet(Indexer,Doc);
-	}else
-	if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "cgi")) {
-		res = UdmExecGet(Indexer,Doc);
-	}
+  UdmBuildHTTPRequest(Doc);
+  
+  /* If mirroring is enabled */
+  if (mirror_period >= 0)
+  {
+    /* 
+      on u_m==0 it returned by mtime from mirrorget
+      but we knew that it should be put in mirror
+    */
+    res= UdmMirrorGET(Indexer, Doc, &Doc->CurURL);
+    if(!res)
+    {
+      UdmLog(Indexer, UDM_LOG_DEBUG, "Taken from mirror");
+      found_in_mirror = 1;
+    }
+  }
+
+  if (!found_in_mirror)
+  {
+    if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "exec"))
+    {
+      res= UdmExecGet(Indexer,Doc);
+    }
+    else if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "cgi"))
+    {
+      res = UdmExecGet(Indexer,Doc);
+    }
 #ifdef HAVE_SQL
-	else
-	if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "htdb")) {
-		res = UdmHTDBGet(Indexer, Doc);
-	}
+    else if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "htdb"))
+    {
+      res= UdmHTDBGet(Indexer, Doc);
+    }
 #endif
 #ifdef USE_FILE
-	else
-	if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "file")) {
-		res = UdmFILEGet(Indexer,Doc);
-	}
+    else if(!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "file"))
+    {
+      res= UdmFILEGet(Indexer,Doc);
+    }
 #endif
 #ifdef USE_NEWS
-	else
-	if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "news"))) {
-		res = UdmNNTPGet(Indexer,Doc);
-	}else
-	if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "nntp"))) {
-		res = UdmNNTPGet(Indexer,Doc);
-	}
+    else if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "news")))
+    {
+      res= UdmNNTPGet(Indexer,Doc);
+    }
+    else if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "nntp")))
+    {
+      res= UdmNNTPGet(Indexer,Doc);
+    }
 #endif
 #ifdef USE_HTTPS
-	else
-	if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "https"))) {
-		res = UdmHTTPSGet(Indexer,Doc);
-	}
+    else if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "https")))
+    {
+      res= UdmHTTPSGet(Indexer,Doc);
+    }
 #endif
 #ifdef USE_HTTP
-	else
-	if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "http")) ||
-	   (!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "ftp") && (proxy))) {
-		res = UdmHTTPGet(Indexer,Doc);
-	}
+    else if((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "http")) ||
+            (!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "ftp") && (proxy)))
+    {
+      res= UdmHTTPGet(Indexer,Doc);
+    }
 #endif
 #ifdef USE_FTP
-	else
-	if ((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "ftp")) && (!proxy)) {
-		res = UdmFTPGet(Indexer,Doc);
-	}
+    else if ((!strcasecmp(UDM_NULL2EMPTY(Doc->CurURL.schema), "ftp")) && (!proxy))
+    {
+      res= UdmFTPGet(Indexer,Doc);
+    }
 #endif
-	}
+  }
 
-	/* Add NULL terminator */
-	Doc->Buf.buf[Doc->Buf.size]='\0';
-	
-	switch(res){
-		case UDM_NET_UNKNOWN:
-			UdmLog(Indexer,UDM_LOG_WARN,"Protocol not supported");
-			UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_NOT_SUPPORTED);
-			break;
-		case UDM_NET_ERROR:
-			UdmLog(Indexer,UDM_LOG_WARN,"Network error");
-			UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
-			break;
-		case UDM_NET_TIMEOUT:
-			UdmLog(Indexer,UDM_LOG_WARN,"Download timeout");
-			UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_GATEWAY_TIMEOUT);
-			break;
-		case UDM_NET_CANT_CONNECT:
-			UdmLog(Indexer,UDM_LOG_WARN,"Can't connect to host %s:%d",Doc->connp.hostname,Doc->connp.port);
-			UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
-			break;
-		case UDM_NET_CANT_RESOLVE:
-			UdmLog(Indexer,UDM_LOG_WARN,"Unknown %shost '%s'",proxy?"proxy ":"",Doc->connp.hostname);
-			UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
-			break;
-		default:
-			if(res<0){	/* No connection */
-				UdmLog(Indexer,UDM_LOG_WARN,"Can't connect to host %s:%d",Doc->connp.hostname,Doc->connp.port);
-				UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
-				break;
-			}
-	}
-	/* Put into mirror if required */
-	if ((mirror_period >= 0) && (!found_in_mirror)){
-		if(UdmMirrorPUT(Indexer, Doc, &Doc->CurURL)){
-			return UDM_ERROR;
-		}
-	}
-	return UDM_OK;
+  /* Add NULL terminator */
+  Doc->Buf.buf[Doc->Buf.size]= '\0';
+  
+  switch(res)
+  {
+    case UDM_NET_UNKNOWN:
+      UdmLog(Indexer,UDM_LOG_WARN,"Protocol not supported");
+      UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_NOT_SUPPORTED);
+      break;
+    case UDM_NET_ERROR:
+      UdmLog(Indexer,UDM_LOG_WARN,"Network error");
+      UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
+      break;
+    case UDM_NET_TIMEOUT:
+      UdmLog(Indexer,UDM_LOG_WARN,"Download timeout");
+      UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_GATEWAY_TIMEOUT);
+      break;
+    case UDM_NET_CANT_CONNECT:
+      UdmLog(Indexer,UDM_LOG_WARN,"Can't connect to host %s:%d",Doc->connp.hostname,Doc->connp.port);
+      UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
+      break;
+    case UDM_NET_CANT_RESOLVE:
+      UdmLog(Indexer,UDM_LOG_WARN,"Unknown %shost '%s'",proxy?"proxy ":"",Doc->connp.hostname);
+      UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
+      break;
+    default:
+      if(res<0)
+      {  /* No connection */
+        UdmLog(Indexer,UDM_LOG_WARN,"Can't connect to host %s:%d",Doc->connp.hostname,Doc->connp.port);
+        UdmCreateErrorResponse(Doc, UDM_HTTP_STATUS_SERVICE_UNAVAILABLE);
+        break;
+      }
+  }
+  
+  /* Put into mirror if required */
+  if ((mirror_period >= 0) && (!found_in_mirror))
+  {
+    if(UdmMirrorPUT(Indexer, Doc, &Doc->CurURL))
+    {
+      return UDM_ERROR;
+    }
+  }
+  return UDM_OK;
 }
