@@ -317,13 +317,6 @@ static UDM_URL_CRD *UdmSearchdGetWordResponse(UDM_AGENT *query,UDM_RESULT *Res,U
 #endif
 				done=1;
 				break;
-		        case UDM_SEARCHD_CMD_PERSITE:
-			        Res->PerSite = (size_t*)UdmMalloc(hdr.len + 1);
-				nrecv = UdmRecvall(cl->searchd, Res->PerSite, hdr.len);
-#ifdef DEBUG_SDP
-				fprintf(stderr, "Received PerSite size=%d nwrd=%d\n",hdr.len,Res->total_found);
-#endif
-				break;
 		        case UDM_SEARCHD_CMD_DATA:
 			        Res->CoordList.Data = (UDM_URLDATA*)UdmMalloc(hdr.len + 1);
 				nrecv = UdmRecvall(cl->searchd, Res->CoordList.Data, hdr.len);
@@ -335,7 +328,6 @@ static UDM_URL_CRD *UdmSearchdGetWordResponse(UDM_AGENT *query,UDM_RESULT *Res,U
 /*				Res->offset = 1;*/
 				break;
 		        case UDM_SEARCHD_CMD_WWL:
-				Res->PerSite = NULL;
 			        if ((wbuf = p = (char *)UdmXmalloc(hdr.len)) != NULL) 
 				  if (UdmRecvall(cl->searchd, wbuf, hdr.len))  {
 				    wwl = (UDM_WIDEWORDLIST *)p;
