@@ -199,6 +199,7 @@ static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
   UdmHTMLTOKInit(&tag);
   for(htok= UdmHTMLToken(Doc->Buf.content, (const char **)&last, &tag) ; htok ;)
   {
+    char *tmp;
     switch(tag.type) {
     case UDM_HTML_COM:
     case UDM_HTML_TAG:
@@ -211,9 +212,21 @@ static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
       ch= *last;
       *last= '\0';
       if (tag.title || tag.script || tag.comment || tag.style)
-        sprintf(HEnd, "%s", UdmHlConvert(NULL, htok, cs, cs));
+      {
+        if ((tmp= UdmHlConvert(NULL, htok, cs, cs)))
+        {
+          memcpy(HEnd, tmp, strlen(tmp) + 1);
+          UdmFree(tmp);
+        }
+      }
       else
-        sprintf(HEnd, "%s", UdmHlConvert(&Res->WWList, htok, cs, cs));
+      {
+        if ((tmp= UdmHlConvert(&Res->WWList, htok, cs, cs)))
+        {
+          memcpy(HEnd, tmp, strlen(tmp) + 1);
+          UdmFree(tmp);
+        }
+      }
       HEnd=UDM_STREND(HEnd);
       *last = ch;
       break;
@@ -502,7 +515,11 @@ int main(int argc, char ** argv)
   }
   
   if (UdmVarListFindInt(&Env->Vars, "cc", 0))
+  {
+    UdmVarListFree(&query_vars);
+    UDM_FREE(query_string);
     return(StoreDocCGI(Agent, Env, &tmpl));
+  }
   
   if (httpd)
     printf("Content-type: text/html; charset=%s\r\n\r\n",bcharset);
