@@ -53,9 +53,10 @@ static void cache_file_name(char *dst,size_t len,UDM_ENV *Conf,UDM_RESULT *Res){
 	size_t bytes;
 /*	UDM_DB *db = Conf->db;*/
 	
-	bytes = udm_snprintf(param, sizeof(param)-1, "%s.%s.%s.%d.%s.%s.%s.%s.%s.%s.%s.%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s-%s",
+	bytes = udm_snprintf(param, sizeof(param)-1, "%s.%s.%s.%s.%d.%s.%s.%s.%s.%s.%s.%s.%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s-%s",
 			     UdmVarListFindStr(&Conf->Vars,"SearchMode","all"),
 			     UdmVarListFindStr(&Conf->Vars, "orig_m", ""),
+			     UdmVarListFindStr(&Conf->Vars, "fl", ""),
 			     UdmVarListFindStr(&Conf->Vars, "wm", ""),
 			     UdmVarListFindInt(&Conf->Vars, "o", 0),
 			     UdmVarListFindStr(&Conf->Vars, "t", ""),
@@ -190,14 +191,8 @@ int __UDMCALL UdmSearchCacheStore(UDM_AGENT * query,UDM_RESULT *Res){
 			
 			write(fd, Res->CoordList.Coords, Res->CoordList.ncoords * sizeof(UDM_URL_CRD));
 			write(fd, Res->CoordList.Data, Res->CoordList.ncoords * sizeof(UDM_URLDATA));
-			if (Res->PerSite) {
-			  write(fd, &Res->total_found, sizeof(Res->total_found));
-			  write(fd, Res->PerSite, Res->CoordList.ncoords * sizeof(size_t));
-			} else {
-			  topcount = 0;
-			  write(fd, &topcount, sizeof(topcount));
-			}
-			
+		        topcount = 0;
+			write(fd, &topcount, sizeof(topcount));
 			close(fd);
 		}else{
 #ifdef DEBUG_CACHE
@@ -297,19 +292,10 @@ int __UDMCALL UdmSearchCacheFind(UDM_AGENT * Agent,UDM_RESULT *Res){
 		close(fd);
 		return(-1);
 	}
-	if (topcount) {
-	  Res->PerSite = (size_t*)UdmMalloc(Res->total_found * sizeof(size_t));
-	  if(-1 == (bytes=read(fd, Res->PerSite, topcount * sizeof(size_t) ))){
-		close(fd);
-		return(-1);
-	  }
-	}
-
 	close(fd);
 	UDM_FREE(Res->CoordList.Coords);
 	Res->CoordList.Coords = wrd;
 	Res->CoordList.Data = dat;
-	
 	Res->num_rows = Res->total_found = Res->CoordList.ncoords; /* ??? was commented, why ? */
 
 #ifdef DEBUG_CACHE
