@@ -52,6 +52,7 @@ static int UdmMySQLInit(UDM_DB *db)
   const char* DBUser= UdmVarListFindStr(&db->Vars,"DBUser",NULL);
   const char* DBPass= UdmVarListFindStr(&db->Vars,"DBPass",NULL);
   const char* DBHost= UdmVarListFindStr(&db->Vars, "DBHost", "localhost");
+  const char* mycnfgroup= UdmVarListFindStr(&db->Vars, "mycnfgroup", "client");
   int DBPort= UdmVarListFindInt(&db->Vars, "DBPort", 0);
   MYSQL *mysql;
   
@@ -63,6 +64,9 @@ static int UdmMySQLInit(UDM_DB *db)
     return UDM_ERROR;
   }
 
+  if (strcasecmp(mycnfgroup, "no"))
+    mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, mycnfgroup);
+  
   if(!(mysql_real_connect(mysql, DBHost, DBUser, DBPass,
   			  db->DBName?db->DBName:"mnogosearch",
   			  (unsigned)DBPort, DBSock, 0)))
