@@ -40,7 +40,7 @@
 #include <oci.h>
 
 #define	MAX_COLS_IN_TABLE	32
-#define MAX_BIND_PARAM		4
+#define MAX_BIND_PARAM		34
 #define BUF_OUT_SIZE		16
 
 /* (C) copyleft 2000 Anton Zemlyanov, az@hotmail.ru */
@@ -160,7 +160,7 @@ static int UdmOracle8InitDB(UDM_DB *db)
 static int oracle_read_blob (UDM_DB *db, OCILobLocator *lobp, UDM_PSTR *items)
 {
   int rc;
-  size_t len;
+  ub4 len;
   char *buf;
   UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
 
@@ -556,7 +556,8 @@ static int UdmOracle8Prepare (UDM_DB *db, const char *query)
 }
 
 
-static int UdmOracle8Bind (UDM_DB *db, int position, void *data, size_t size, int type)
+static int UdmOracle8Bind (UDM_DB *db, int position,
+                           void *data, int size, int type)
 {
   int _;
   UDM_ORACLE_DB *ob = (UDM_ORACLE_DB *)db->specific;
