@@ -65,11 +65,42 @@ static int UdmPgSQLInitDB(UDM_DB *db)
   return UDM_OK;
 }
 
+
+#define udm_oct2int(ch) ((ch) - '0')
+#define udm_isoct(ch)   ((ch) >= '0' && (ch) <= '7')
+
+static size_t PgUnescape(char *dst, const char *src, size_t length)
+{
+  char *dst0= dst;
+  const char *srcend= src + length;
+  for ( ; src < srcend; dst++, src++)
+  {
+    if (*src == '\\' && src + 4 <= srcend &&
+        udm_isoct(src[1]) && udm_isoct(src[2]) && udm_isoct(src[3]))
+    {
+      *dst= udm_oct2int(src[1]) * 64 +
+            udm_oct2int(src[2]) * 8 +
+            udm_oct2int(src[3]);
+      src+= 3;
+    }
+    else if (*src == '\\' && src[1] == '\\')
+    {
+      *dst= '\\';
+      src++;
+    }
+    else
+      *dst= *src;
+  }
+  return dst - dst0;
+}
+
+
 static int UdmPgSQLQuery(UDM_DB *db, UDM_SQLRES *res, const char *q)
 {
   size_t i;
   PGresult *PGres;
   PGconn *pgsql;
+  int have_bytea= 0;
   
   if (res)
   {
@@ -136,7 +167,38 @@ static int UdmPgSQLQuery(UDM_DB *db, UDM_SQLRES *res, const char *q)
   res->nRows=(size_t)PQntuples(res->pgsqlres);
   res->Fields=(UDM_SQLFIELD*)UdmMalloc(res->nCols*sizeof(UDM_SQLFIELD));
   for(i=0;i<res->nCols;i++)
+  {
     res->Fields[i].sqlname = (char*)UdmStrdup(PQfname(res->pgsqlres,(int)i));
+    if (PQftype(res->pgsqlres, (int) i) == 17)
+      have_bytea++;
+  }
+  if (have_bytea)
+  {
+    size_t row;
+    res->Items= (UDM_PSTR*)UdmMalloc(res->nRows * res->nCols * sizeof(UDM_PSTR));
+    for (row= 0; row < res->nRows; row++)
+    {
+      size_t col, offs= res->nCols * row;
+      for(col= 0; col < res->nCols; col++)
+      {
+        UDM_PSTR *I= &res->Items[offs + col];
+        const char *ptr= PQgetvalue(res->pgsqlres,(int) row, (int) col);
+        size_t len= PQgetlength(res->pgsqlres, (int) row, (int )col);
+        I->val= (char*) UdmMalloc(len + 1);
+        if (PQftype(res->pgsqlres, (int) col) == 17)
+        {
+          I->len= PgUnescape(I->val, ptr, len);
+          I->val[I->len]= '\0';
+        }
+        else
+        {
+          memcpy(I->val, ptr, len);
+          I->val[len]= '\0';
+          I->len= len;
+        }
+      }
+    }
+  }
   return UDM_OK;
 }
 
@@ -158,8 +220,8 @@ static int UdmPgSQLCommit(UDM_DB *db)
   return UdmPgSQLQuery(db,NULL,"END WORK");
 }
 
-static
-int UdmPgSQLFetchRow (UDM_DB *db, UDM_SQLRES *res, UDM_PSTR *buf)
+static int
+UdmPgSQLFetchRow(UDM_DB *db, UDM_SQLRES *res, UDM_PSTR *buf)
 {
   size_t j;
 
@@ -169,7 +231,12 @@ int UdmPgSQLFetchRow (UDM_DB *db, UDM_SQLRES *res, UDM_PSTR *buf)
   for (j = 0; j < res->nCols; j++)
   {
     buf[j].val= PQgetvalue(res->pgsqlres,(int)res->curRow,(int)(j));
-    buf[j].len= 0;
+    buf[j].len= PQgetlength(res->pgsqlres, res->curRow, j);
+    if (PQftype(res->pgsqlres, (int) j) == 17)
+    {
+      buf[j].len= PgUnescape(buf[j].val, buf[j].val, buf[j].len);
+      buf[j].val[buf[j].len]= '\0';
+    }
   }
   res->curRow++;
   return(UDM_OK);
