@@ -41,12 +41,56 @@
 #include "udm_utils.h"
 #include "udm_vars.h"
 #include "udm_sqldbms.h"
-
 #include "udm_xmalloc.h"
+
+#if HAVE_IODBC
+#include <sql.h>
+#include <sqlext.h>
+#endif
+
+#if HAVE_EASYSOFT
+#include <sql.h>
+#include <sqlext.h>
+#endif
+
+#if HAVE_VIRT
+#include <iodbc.h>
+#include <isql.h>
+#include <isqlext.h>
+#endif
+
+#if HAVE_UNIXODBC
+#include <sql.h>
+#include <sqlext.h>
+#endif
+
+#if HAVE_SAPDB
+#include <WINDOWS.H>
+#include <sql.h>
+#include <sqlext.h>
+#endif
+
+#if HAVE_SOLID
+#include <cli0cli.h>
+#endif
+
+#if HAVE_DB2
+#include <sqlcli1.h>
+#endif
+
+#if HAVE_CTLIB
+#include <ctpublic.h>
+#endif
+
+#if HAVE_SQLITE
+#include <sqlite.h>
+#endif
+
 #ifdef WIN32
 #include <process.h>
 #endif
 
+
 void DecodeHexStr (char *src, UDM_PSTR *dst, size_t size)
 {
   char p1;
@@ -111,6 +155,20 @@ static int udb_free_result(UDM_SQLRES *res){
 /************************** ODBC ***********************************/
 #if (HAVE_ODBC)
 
+typedef struct udm_odbc_conn_st
+{
+#if (HAVE_IODBC || HAVE_UNIXODBC || HAVE_SOLID || HAVE_VIRT || HAVE_EASYSOFT || HAVE_SAPDB)
+  HDBC  hDbc;
+  HENV  hEnv;
+  HSTMT hstmt;
+#endif
+#if HAVE_DB2
+  SQLHANDLE hDbc;
+  SQLHANDLE hEnv;
+  SQLHANDLE hstmt;
+#endif
+} UDM_ODBC_CONN;
+
 
 #define SQL_OK(rc)  ((rc==SQL_SUCCESS)||(rc==SQL_SUCCESS_WITH_INFO))
 
@@ -122,12 +180,13 @@ static int UdmODBCDisplayError(UDM_DB *db, const char *prefix)
   SWORD length;
   RETCODE rc=SQL_SUCCESS;
   size_t len= strlen(prefix);
-  
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
+
   strcpy(db->errstr, prefix);
   
   while (1)
   {   
-    rc = SQLError(db->hEnv, db->hDbc, db->hstmt, szSqlState,
+    rc = SQLError(sdb->hEnv, sdb->hDbc, sdb->hstmt, szSqlState,
                   &naterr,szErrMsg,sizeof(szErrMsg)-1,&length);
     if ((SQL_SUCCESS != rc) && (SQL_SUCCESS_WITH_INFO != rc))
       break;
@@ -155,10 +214,11 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
   SWORD iResColumns;
   SDWORD iRowCount;
   static char  bindbuf[(int)(64L * 1024L - 16L)];
-  
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
+
   if(!strcmp(sqlstr,"COMMIT"))
   {
-    rc=SQLTransact(db->hEnv,db->hDbc,SQL_COMMIT);
+    rc=SQLTransact(sdb->hEnv, sdb->hDbc, SQL_COMMIT);
     if(!SQL_OK(rc))
     {
       db->errcode=1;
@@ -171,14 +231,14 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
     }
   }
   
-  rc=SQLAllocStmt(db->hDbc, &(db->hstmt));
+  rc=SQLAllocStmt(sdb->hDbc, &sdb->hstmt);
   if (!SQL_OK(rc))
   {
     db->errcode=1;
     return UDM_ERROR;
   }
   
-  rc=SQLExecDirect(db->hstmt,(SQLCHAR *)sqlstr, SQL_NTS);
+  rc=SQLExecDirect(sdb->hstmt,(SQLCHAR *)sqlstr, SQL_NTS);
   if (!SQL_OK(rc))
   {
     if(rc==SQL_NO_DATA) goto ND;
@@ -186,7 +246,7 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
     return UDM_ERROR; 
   }
   
-  rc=SQLNumResultCols(db->hstmt, &iResColumns);
+  rc=SQLNumResultCols(sdb->hstmt, &iResColumns);
   if(!SQL_OK(rc))
   {
     db->errcode=1;
@@ -195,7 +255,7 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
   
   if(!iResColumns)
   {
-    rc=SQLRowCount(db->hstmt, &iRowCount);
+    rc=SQLRowCount(sdb->hstmt, &iRowCount);
     if (!SQL_OK(rc))
     {
       db->errcode=1;
@@ -213,23 +273,27 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
     result->nCols = iResColumns;
     result->Items = NULL;
 
+    result->Fields=(UDM_SQLFIELD*)UdmMalloc(result->nCols*sizeof(UDM_SQLFIELD));
+    bzero(result->Fields, result->nCols*sizeof(UDM_SQLFIELD));
+
     for(col=0; col < iResColumns; col++)
     {
-      SQLDescribeCol(db->hstmt, col+1,
+      SQLDescribeCol(sdb->hstmt, col+1,
                      Field[col].Name, 
-                     (SQLSMALLINT) sizeof(Field[col].Name),
+                     (SQLSMALLINT) sizeof(Field[col].Name - 1),
                      &Field[col].NameLen,
                      &Field[col].Type,
                      &Field[col].Size,
                      &Field[col].Scale,
                      &Field[col].Nullable);
+      result->Fields[col].sqlname= (char*)UdmStrdup(Field[col].Name);
     }
 
     rc= SQL_NO_DATA_FOUND;
     for (mRows=0, res_count=0;(db->res_limit?(res_count<db->res_limit):1);res_count++)
     {
       int i;
-      rc=SQLFetch(db->hstmt);
+      rc=SQLFetch(sdb->hstmt);
       if (!SQL_OK(rc))
       {
         if (rc != SQL_NO_DATA_FOUND)
@@ -252,7 +316,21 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
         if (Field[i].Type == SQL_DOUBLE)
           get_type= SQL_C_DOUBLE;
 
-        rc= SQLGetData(db->hstmt,i+1,get_type,bindbuf,sizeof(bindbuf),&pcbValue);
+        /* PostgreSQL bytea */
+        if (Field[i].Type == SQL_VARBINARY && db->DBType == UDM_DB_PGSQL)
+          get_type= SQL_C_BINARY;
+
+#ifdef HAVE_UNIXODBC
+        /*
+           For some weird reasons, Sybase native (i.e. not FreeTDS)
+           ODBC driver in conjunction with unixODBC always returns 4
+           in pcbValue if get_type is SQL_CHAR. Let's fetch as SQL_C_SLONG.
+        */
+        if (Field[i].Type == SQL_INTEGER && db->DBType == UDM_DB_SYBASE)
+          get_type= SQL_C_SLONG;
+#endif
+
+        rc= SQLGetData(sdb->hstmt,i+1,get_type,bindbuf,sizeof(bindbuf),&pcbValue);
         if ((rc != SQL_SUCCESS) || (pcbValue == SQL_NULL_DATA))
         {
           p= "";
@@ -272,11 +350,20 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
             memcpy(&num, bindbuf, sizeof(num));
             pcbValue= sprintf(bindbuf, "%f", num);
           }
+          else if (get_type == SQL_C_SLONG)
+          {
+            long int num;
+            memcpy(&num, bindbuf, sizeof(num));
+            pcbValue= sprintf(bindbuf, "%ld", num);
+          }
           p= bindbuf;
         }
-
         /* if(p)UdmRTrim(p," "); */
         if (db->DBType != UDM_DB_ORACLE8 &&
+            db->DBType != UDM_DB_PGSQL &&
+#ifdef HAVE_UNIXODBC
+            db->DBType != UDM_DB_SYBASE &&
+#endif
             (Field[i].Type == SQL_BINARY ||
              Field[i].Type == SQL_VARBINARY ||
              Field[i].Type == SQL_LONGVARBINARY ||
@@ -298,7 +385,7 @@ static int execDB(UDM_DB*db,UDM_SQLRES *result, const char *sqlstr)
   }
 ND:
 
-  SQLFreeStmt(db->hstmt, SQL_DROP);
+  SQLFreeStmt(sdb->hstmt, SQL_DROP);
   db->res_limit=0;
   db->errcode=0;
   return UDM_OK;
@@ -312,7 +399,13 @@ static int UdmODBCInitDB(UDM_DB *db)
   const char* DBHost= UdmVarListFindStr(&db->Vars, "DBHost", "localhost");
   int DBPort= UdmVarListFindInt(&db->Vars, "DBPort", 0);
   UDM_SQLRES SQLRes;
-  
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) UdmMalloc(sizeof(UDM_ODBC_CONN));
+  db->specific= (void*) sdb;
+
+  sdb->hDbc=  SQL_NULL_HDBC;
+  sdb->hEnv=  SQL_NULL_HENV;
+  sdb->hstmt= SQL_NULL_HSTMT;
+
 #if (HAVE_SOLID)
   udm_snprintf(DSN, sizeof(DSN)-1, "tcp %s %d", DBHost, DBPort?DBPort:1313);
 #elif (HAVE_SAPDB)
@@ -321,13 +414,13 @@ static int UdmODBCInitDB(UDM_DB *db)
   strncpy(DSN, db->DBName?db->DBName:"", sizeof(DSN)-1);
 #endif
   
-  db->errcode = SQLAllocEnv( &(db->hEnv) );
+  db->errcode = SQLAllocEnv(&sdb->hEnv);
   if( SQL_SUCCESS != db->errcode )return -2;
-  db->errcode = SQLAllocConnect( db->hEnv, &(db->hDbc) );
+  db->errcode = SQLAllocConnect(sdb->hEnv, &sdb->hDbc);
   if( SQL_SUCCESS != db->errcode )return -3;
-  db->errcode = SQLSetConnectOption( db->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
+  db->errcode = SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
   if( SQL_SUCCESS != db->errcode )return -4;
-  db->errcode = SQLConnect( db->hDbc, (SQLCHAR *)DSN, SQL_NTS, DBUser, SQL_NTS, DBPass, SQL_NTS);
+  db->errcode = SQLConnect(sdb->hDbc, (SQLCHAR *)DSN, SQL_NTS, DBUser, SQL_NTS, DBPass, SQL_NTS);
   if( !SQL_OK(db->errcode)) return(-5);
   else db->errcode=0;
   db->connected=1;
@@ -340,23 +433,22 @@ static void UdmODBCClose(UDM_DB *db)
 {
   if(db->connected)
   {
-    db->connected=0;
-    db->errcode = SQLTransact( db->hEnv, db->hDbc, SQL_COMMIT);
-    if( SQL_SUCCESS != db->errcode )
-      return;
-    db->errcode = SQLDisconnect( db->hDbc );
-    if( SQL_SUCCESS != db->errcode )
-      return;
-    db->errcode = SQLFreeConnect( db->hDbc );
-    if( SQL_SUCCESS != db->errcode )
-      return;
-    else
-      db->hDbc = SQL_NULL_HDBC;
-    db->errcode = SQLFreeEnv( db->hEnv );
-    if( SQL_SUCCESS != db->errcode )
-      return;
-    else
-      db->hEnv = SQL_NULL_HENV;
+    UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
+    db->errcode= SQLTransact(sdb->hEnv, sdb->hDbc, SQL_COMMIT);
+    if(SQL_SUCCESS != db->errcode)
+      goto ret;
+    db->errcode= SQLDisconnect(sdb->hDbc );
+    if(SQL_SUCCESS != db->errcode)
+      goto ret;
+    db->errcode= SQLFreeConnect(sdb->hDbc);
+    if(SQL_SUCCESS != db->errcode)
+      goto ret;
+    db->errcode= SQLFreeEnv(sdb->hEnv);
+    if(SQL_SUCCESS != db->errcode)
+      goto ret;
+ret:
+    db->connected= 0;
+    UDM_FREE(db->specific);
   }
 }
 
@@ -394,11 +486,14 @@ static int UdmODBCQuery(UDM_DB *db,UDM_SQLRES *res, const char *qbuf)
       db->errcode=0;
       rc= UDM_OK;
     }
+#if 0
+    /* Cannot use S1000: it's returned by unixODBC in many cases */
     else if(strstr(db->errstr,"[SQLSTATE:S1000]"))
     { /* Oracle 8i */
       db->errcode=0;
       rc= UDM_OK;
     }
+#endif
     else if(strstr(db->errstr,"uplicat")) /* PgSQL,MySQL*/
     {
       db->errcode=0;
@@ -418,44 +513,67 @@ static int UdmODBCQuery(UDM_DB *db,UDM_SQLRES *res, const char *qbuf)
     {
       db->errcode=1;
     }
-    SQLFreeStmt(db->hstmt, SQL_DROP);
+    {
+      UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
+      SQLFreeStmt(sdb->hstmt, SQL_DROP);
+      sdb->hstmt= SQL_NULL_HSTMT;
+    }
+    
   }
   return rc;
 }
 
 static int UdmODBCBegin(UDM_DB *db)
 {
-  db->errcode = SQLSetConnectOption( db->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
-  if( SQL_SUCCESS != db->errcode )return UDM_ERROR;
+  UDM_ODBC_CONN *sdb;
+  if(!db->connected)
+  {
+    UdmODBCInitDB(db);
+    if(db->errcode)
+    {
+      UdmODBCDisplayError(db,"");
+      return UDM_ERROR;
+    }
+    else
+    {
+      db->connected=1;
+    }
+  }
+  sdb= (UDM_ODBC_CONN*) db->specific;
+  db->errcode= SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
+  if(SQL_SUCCESS != db->errcode)return UDM_ERROR;
   return UDM_OK;
 }
 
 
 static int UdmODBCCommit(UDM_DB *db)
 {
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
   int rc;
   if (UDM_OK!=(rc=UdmODBCQuery(db,NULL,"COMMIT")))
     return UDM_ERROR;
-  db->errcode = SQLSetConnectOption( db->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
-  if( SQL_SUCCESS != db->errcode )
+  db->errcode= SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
+  if(SQL_SUCCESS != db->errcode)
     return UDM_ERROR;
   return UDM_OK;
 }
 
 static int UdmODBCPrepare (UDM_DB *db, const char *query)
 {
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
   int rc;
   db->errcode = 0;
   db->errstr[0] = 0;
-  rc = SQLAllocStmt(db->hDbc, &db->hstmt);
-  if (! SQL_OK(rc))
+  rc= SQLAllocStmt(sdb->hDbc, &sdb->hstmt);
+
+  if (!SQL_OK(rc))
   {
     db->errcode = rc;
     UdmODBCDisplayError(db,"UdmODBCPrepare: ");
     return(UDM_ERROR);
   }
-  rc = SQLPrepare(db->hstmt, query, SQL_NTS);
-  if (! SQL_OK(rc))
+  rc= SQLPrepare(sdb->hstmt, query, SQL_NTS);
+  if (!SQL_OK(rc))
   {
     db->errcode = rc;
     UdmODBCDisplayError(db, "UdmODBCPrepare: ");
@@ -464,26 +582,66 @@ static int UdmODBCPrepare (UDM_DB *db, const char *query)
   return(UDM_OK);
 }
 
-static size_t save_size;
-static void *save_data;
 
-static int UdmODBCBind (UDM_DB *db, int position, void *data, size_t size, int type)
+static int
+UdmSQLType2ODBCType(int udm_type)
 {
-  SQLINTEGER s = (SQLINTEGER)size;
+  switch (udm_type)
+  {
+    case UDM_SQLTYPE_LONGVARBINARY: return SQL_LONGVARBINARY;
+    case UDM_SQLTYPE_LONGVARCHAR  : return SQL_LONGVARCHAR;
+    case UDM_SQLTYPE_VARCHAR      : return SQL_VARCHAR;
+    default                       : break;
+  }
+  return SQL_UNKNOWN_TYPE;
+}
+
+
+static int
+UdmSQLParamLen2ODBC(int udm_len)
+{
+  switch (udm_len)
+  {
+    case UDM_SQL_NULL_DATA    : return SQL_NULL_DATA;
+    case UDM_SQL_DATA_AT_EXEC : return SQL_DATA_AT_EXEC;
+    case UDM_SQL_NTS          : return SQL_NTS;
+  }
+  return udm_len;
+}
+
+
+typedef struct udm_odbc_bind_st
+{
+  SQLINTEGER size;
+  void *data;
+} UDM_ODBC_BINDBUF;
+
+static UDM_ODBC_BINDBUF BindBuf[64];
+
+static int UdmODBCBind (UDM_DB *db, int position,
+                        void *data, int size, int type)
+{
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
   int rc;
-  save_size= size;
-  save_data= data;
+  BindBuf[position].size= UdmSQLParamLen2ODBC(size);
+  BindBuf[position].data= data;
+  
+  type= UdmSQLType2ODBCType(type);
   db->errcode = 0;
   db->errstr[0] = 0;
-  rc = SQLBindParameter(db->hstmt, position, SQL_PARAM_INPUT, SQL_C_DEFAULT, type, 0, 0, data, size, &s);
+
+  rc= SQLBindParameter(sdb->hstmt, position, SQL_PARAM_INPUT, SQL_C_DEFAULT,
+                       type, 0 /* Column size */, 0 /* Decimal digits */,
+                       /*&BindBuf[position]*/ data, size < 0 ? 0 : size,
+                       &BindBuf[position].size);
 
 #ifdef WIN32  
-  s= SQL_DATA_AT_EXEC;
+/* s_size= SQL_DATA_AT_EXEC; */
 #endif
 
-  if (! SQL_OK(rc))
+  if (!SQL_OK(rc))
   {
-    db->errcode = rc;
+    db->errcode= rc;
     UdmODBCDisplayError(db, "UdmOBDCBind: ");
     return(UDM_ERROR);
   }
@@ -492,29 +650,37 @@ static int UdmODBCBind (UDM_DB *db, int position, void *data, size_t size, int t
 
 static int UdmODBCExec (UDM_DB *db)
 {
-  int rc;
+  UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
+  int rc, put_data= 0;
+  UDM_ODBC_BINDBUF *Buf;
+
   db->errcode = 0;
   db->errstr[0] = 0;
-  rc= SQLExecute(db->hstmt);
+  rc= SQLExecute(sdb->hstmt);
 
 #ifdef WIN32
-  if (rc == SQL_NEED_DATA)
-  {
-    void *iWhatParam;
-    rc= SQLParamData(db->hstmt, &iWhatParam);
-    if (rc == SQL_NEED_DATA)
-    {
-      rc= SQLPutData(db->hstmt, save_data, save_size);
-    }
-    rc= SQLParamData(db->hstmt, &iWhatParam);
-  }
+  put_data= 1;
 #endif
-  if (! SQL_OK(rc))
+
+  if (!put_data || rc != SQL_NEED_DATA)
+    goto check_err;
+
+  for (rc= SQLParamData(sdb->hstmt, (void*)&Buf);
+       rc == SQL_NEED_DATA;
+       rc= SQLParamData(sdb->hstmt, (void*)&Buf))
+  {
+    if (SQL_ERROR == (rc= SQLPutData(sdb->hstmt, Buf->data, Buf->size)))
+      goto check_err;
+  }
+
+check_err:
+
+  if (!SQL_OK(rc))
   {
     UdmODBCDisplayError(db, "UdmODBCExec: ");
     db->errcode= rc;
   }
-  SQLFreeStmt(db->hstmt, SQL_DROP);
+  SQLFreeStmt(sdb->hstmt, SQL_DROP);
   return SQL_OK(rc) ? UDM_OK : UDM_ERROR;
 }
 
@@ -597,7 +763,7 @@ ex_servermsg_cb(CS_CONTEXT *context,CS_CONNECTION *connection,CS_SERVERMSG *srvm
   if (srvmsg->proclen > 0){fprintf(stderr, " Procedure '%s'\n", srvmsg->proc);}
   */
   
-  /*if (srvmsg->msgnumber != 3621)*/ /* for SYBASE*/
+  if (srvmsg->msgnumber != 3621) /* for SYBASE*/
   if(srvmsg->severity>1)
     udm_snprintf(sybsrvmsg, sizeof(sybsrvmsg)-1, "Message number: %ld, Severity %ld, State: %ld, Line: %ld, Msg: '%s'",(long)srvmsg->msgnumber, (long)srvmsg->severity, (long)srvmsg->state, (long)srvmsg->line,srvmsg->text);
   
@@ -700,7 +866,8 @@ static int UdmCTLIBInitDB(UDM_DB *db)
   const char* DBPass= UdmVarListFindStr(&db->Vars,"DBPass",NULL);
   const char* DBHost= UdmVarListFindStr(&db->Vars, "DBHost", NULL);
   
-  db->specific= malloc(sizeof(sizeof(UDM_CTLIB))); 
+  db->specific= malloc(sizeof(UDM_CTLIB)); 
+  bzero(db->specific, sizeof(UDM_CTLIB));
   ct= (UDM_CTLIB*) db->specific;
   
   retcode = cs_ctx_alloc(CS_VERSION_100, &ct->ctx);
@@ -710,7 +877,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
     db->errcode=1;
     goto unlock_ex;
   }
-  
+
   retcode = ct_init(ct->ctx, CS_VERSION_100);
   if (retcode != CS_SUCCEED)
   {
@@ -738,7 +905,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode == CS_SUCCEED)
   {
     retcode = ct_callback(ct->ctx, NULL, CS_SET, CS_SERVERMSG_CB,(CS_VOID *)ex_servermsg_cb);
@@ -748,7 +915,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode == CS_SUCCEED)
   {
     retcode = ct_config(ct->ctx, CS_SET, CS_NETIO, &netio_type, CS_UNUSED, NULL);
@@ -758,7 +925,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode != CS_SUCCEED)
   {
     ct_exit(ct->ctx, CS_FORCE_EXIT);
@@ -767,17 +934,17 @@ static int UdmCTLIBInitDB(UDM_DB *db)
     db->errcode=1;
     goto unlock_ex;
   }
-  
+
   if (retcode==CS_SUCCEED)
     retcode = ct_con_alloc(ct->ctx, &ct->conn);
-  
+
   if (retcode != CS_SUCCEED)
   {
     udm_snprintf(db->errstr,sizeof(db->errstr)-1,"ct_con_alloc failed: %s",sybsrvmsg);
     db->errcode=1;
     goto unlock_ex;
   }
-  
+
   if (retcode == CS_SUCCEED && DBUser != NULL)
   {
     if ((retcode = ct_con_props(ct->conn, CS_SET, CS_USERNAME, DBUser, CS_NULLTERM, NULL)) != CS_SUCCEED)
@@ -786,7 +953,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode == CS_SUCCEED && DBPass != NULL)
   {
     if ((retcode = ct_con_props(ct->conn, CS_SET, CS_PASSWORD, DBPass, CS_NULLTERM, NULL)) != CS_SUCCEED)
@@ -795,7 +962,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode == CS_SUCCEED)
   {
     if ((retcode = ct_con_props(ct->conn, CS_SET, CS_APPNAME, "indexer", CS_NULLTERM, NULL)) != CS_SUCCEED)
@@ -804,7 +971,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode == CS_SUCCEED)
   {
     len = DBHost ? CS_NULLTERM : 0;
@@ -815,7 +982,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
       db->errcode=1;
     }
   }
-  
+
   if (retcode != CS_SUCCEED)
   {
     ct_con_drop(ct->conn);
@@ -823,7 +990,7 @@ static int UdmCTLIBInitDB(UDM_DB *db)
     db->errcode=1;
     goto unlock_ex;
   }
-  
+
   db->connected=1;
   cmdbuf = (CS_CHAR *) UdmMalloc(1024);
   sprintf(cmdbuf, "use %s\n", db->DBName);
@@ -988,14 +1155,16 @@ int ex_fetch_data(UDM_DB *db, UDM_SQLRES *res, CS_COMMAND *cmd)
       break;
     }
 
-    /* Alloc data and remember the original datatype */
-    coldata[i].value = (CS_CHAR *)UdmMalloc((size_t)datafmt[i].maxlength + 1);
+    /* Remember the original datatype */
     coldata[i].datatype= datafmt[i].datatype;
-    
+
     datafmt[i].maxlength = ex_display_dlen(&datafmt[i]) + 1;
     datafmt[i].datatype = CS_CHAR_TYPE;
     datafmt[i].format   = CS_FMT_NULLTERM;
     
+    /* Allocate data */
+    coldata[i].value = (CS_CHAR *)UdmMalloc((size_t)datafmt[i].maxlength);
+    
     retcode = ct_bind(cmd, (i + 1), &datafmt[i],coldata[i].value, &coldata[i].valuelen,&coldata[i].indicator);
     
     if (retcode != CS_SUCCEED)
@@ -1424,7 +1593,7 @@ char * UdmDBEscStr(int DBType,char *to,const char *from,size_t len)
 {
   char *s=to;
 
-  if (DBType == UDM_DB_ORACLE8  ||
+  if (DBType == UDM_DB_ORACLE8 || DBType == UDM_DB_SYBASE ||
       DBType == UDM_DB_MSSQL || DBType == UDM_DB_DB2 ||
       DBType == UDM_DB_IBASE || DBType == UDM_DB_SAPDB ||
       DBType == UDM_DB_SQLITE || DBType == UDM_DB_ACCESS ||
@@ -1521,7 +1690,7 @@ size_t UdmSQLNumCols(UDM_SQLRES * res)
 const char * __UDMCALL UdmSQLValue(UDM_SQLRES * res,size_t i,size_t j){
 
 #if HAVE_PGSQL
-  if(res->db->DBDriver==UDM_DB_PGSQL)
+  if(res->db->DBDriver==UDM_DB_PGSQL && !res->Items)
     return(PQgetvalue(res->pgsqlres,(int)(i),(int)(j)));
 #endif
   
@@ -1598,12 +1767,8 @@ size_t UdmSQLLen(UDM_SQLRES * res,size_t i,size_t j)
 {
   size_t offs=res->nCols*i+j;
 #if HAVE_PGSQL
-  /* TODO: This should be fixed */
-  if(res->db->DBDriver==UDM_DB_PGSQL)
-  {
-    const char *val= PQgetvalue(res->pgsqlres,(int)(i),(int)(j));
-    return val ? strlen(val) : 0;
-  }
+  if(res->db->DBDriver==UDM_DB_PGSQL && !res->Items)
+    return PQgetlength(res->pgsqlres, i, j);
 #endif
   return res->Items[offs].len;
 }
