@@ -83,6 +83,7 @@ static int *UdmGetExcerptSourceCachedCopy (UDM_AGENT *Agent,
   UDM_CHARSET *dcs= UdmGetCharSet(UdmVarListFindStr(&Doc->Sections, "Charset", "iso-8859-1"));
   const char *CachedCopy= UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL);
   const char *content_type= UdmVarListFindStr(&Doc->Sections, "Content-Type", NULL);
+  const char *deftype=  UdmVarListFindStr(&Agent->Conf->Vars,"DefaultContentType",NULL);
   UDM_HTMLTOK tag;
   const char *htok, *last;
   int hlstop= UdmVarListFindBool(&Agent->Conf->Vars, "ExcerptStopword", 1);
@@ -122,11 +123,13 @@ static int *UdmGetExcerptSourceCachedCopy (UDM_AGENT *Agent,
   if (!content_type ||
       (strcmp(content_type, "text/plain") &&
        strcmp(content_type, "text/xml") &&
+       strcmp(content_type, "text/vnd.wap.wml") &&
        strcmp(content_type, "text/html")))
-    content_type= UdmGuessContentType(Source, zstream.total_out);
+    content_type= UdmGuessContentType(Source, zstream.total_out, deftype);
   UdmDSTRInit(&buf, 1024);
 
-  if (!strcmp(content_type, "text/xml"))
+  if (!strcmp(content_type, "text/xml") ||
+      !strcmp(content_type, "text/vnd.wap.wml"))
   {
     UdmHTMLTOKInit(&tag);
     htok= UdmHTMLToken(Source, &last, &tag);
