@@ -331,38 +331,18 @@ static size_t PrintTextTemplate(UDM_AGENT *A, FILE *stream, char *dst,
   return dlen;
 }
 
-static char * GetHtmlTok(const char * src,const char ** lt)
-{
-  char * res;
-  size_t len;
-  if((!src)&&!(src=*lt))return(NULL);
-  if(*src=='<')
-  {
-    /* Find ">" and skip "<" */
-    if((*lt=strchr(src,'>')))(*lt)++;
-  }
-  else
-  {
-    /* Find tag beginning */
-    *lt=strchr(src,'<');
-  }
-  
-  if(!(*lt))
-  {
-    /* Last token */
-    res = (char*)UdmStrdup(src);
-  }
-  else
-  {
-    /* Token in the middle */
-    len=(*lt)-src;
-    res = (char*)UdmMalloc(len + 2);
-    strncpy(res,src,len);
-    res[len]='\0';
-  }
-  return(res);
-}
 
+static char *
+GetHtmlTok(const char *src, const char **last, UDM_HTMLTOK *t)
+{
+  const char *beg= UdmHTMLToken(src, last, t);
+  if (beg)
+  {
+    size_t length= last[0] - beg;
+    return UdmStrndup(beg, length);
+  }
+  return NULL;
+}
 
 #define UDM_TMPL_IF		1
 #define UDM_TMPL_IFLIKE		2
@@ -558,6 +538,8 @@ static int TemplateFunc2(UDM_TEMPLATE *T)
       }
       else
         vurl[0]= '\0';
+      M.pattern= 0;
+      UdmMatchFree(&M);
     }
     else
       vurl[0]= '\0';
@@ -806,7 +788,7 @@ static size_t PrintTagTemplate(UDM_AGENT *Agent,FILE *stream,
     var = UdmVarListFindWithValue(vars, UdmTrim(vname, "$()"), value ? value:"");
   }
 
-  sprintf(UDM_STREND(opt), "%s>", var ? " selected":"");
+  sprintf(UDM_STREND(opt), "%s>", var ? " selected=\"selected\"":"");
 
   if (vname) { UDM_FREE(vname); }
   if (value) { UDM_FREE(value); }
@@ -1148,8 +1130,8 @@ static int HtmlTemplatePrgAddInvokeMethod(UDM_TMPL_PRG *prg,
   args[1]= UdmVarListFindStr(&attr, "Content", "");
   args[2]= UdmVarListFindStr(&attr, "Result", "");
   args[3]= UdmVarListFindStr(&attr, "Match", "");
-  
-  if (tag.ntoks > 1)
+
+  if (tag.ntoks > 1 && args[0][0])
   {
     i.cmdnum= UDM_TMPL_METHOD;
     i.cmd= &udm_tmpl_method;
@@ -1161,6 +1143,11 @@ static int HtmlTemplatePrgAddInvokeMethod(UDM_TMPL_PRG *prg,
     i.jump= 0;
     HtmlTemplatePrgAdd(prg, &i);
   }
+  else
+  {
+    /* Just print special tags:  <!DOCTYPE ... > */
+    HtmlTemplatePrgAdd1Arg(prg, &udm_tmpl_txt, template);
+  }
   UdmVarListFree(&attr);
   return UDM_OK;
 }
@@ -1174,7 +1161,7 @@ static int HtmlTemplateCompileCommand(UDM_TMPL_PRG *prg,
   UDM_VARLIST  attr;
   UDM_TMPL_COMPILE_STACK_ITEM sitem;
   const char *hlast=NULL;
-  const char *args[3];
+  const char *args[4];
   int rc= UDM_OK;
   
   UdmHTMLTOKInit(&tag);
@@ -1314,13 +1301,14 @@ static int HtmlTemplateCompile(UDM_TMPL_PRG *prg, const char *template)
   char *tok;
   UDM_TMPL_COMPILE_STACK st;
   int rc= UDM_OK;
-  
+  UDM_HTMLTOK HtmlTok;
+  UdmHTMLTOKInit(&HtmlTok);
   CompileStackInit(&st);
   
-  tok= GetHtmlTok(template, &lt);
+  tok= GetHtmlTok(template, &lt, &HtmlTok);
   while (tok)
   {
-    if (tok[0]=='<' && tok[1] == '!')
+    if (tok[0]=='<' && tok[1] == '!' && isalpha(tok[2]))
     {
       UDM_TMPL_CMD *cmd;
       if ((cmd= HtmlTemplateCmd(tok+2)))
@@ -1345,7 +1333,7 @@ static int HtmlTemplateCompile(UDM_TMPL_PRG *prg, const char *template)
     UDM_FREE(tok);
     if (rc != UDM_OK)
       break;
-    tok=GetHtmlTok(NULL,&lt);
+    tok=GetHtmlTok(NULL, &lt, &HtmlTok);
   }
   CompileStackFree(&st);
   return rc;
