@@ -30,7 +30,7 @@ __C_LINK int __UDMCALL udm_mb_wc_8bit(UDM_CONV *conv, UDM_CHARSET *cs,int *wc,
 
   conv->icodes = conv->ocodes = 1;
 
-  if (*str == '&' && (conv->flags & UDM_RECODE_HTML))
+  if (*str == '&' && (conv->flags & UDM_RECODE_HTML_SPECIAL))
   {
     if ((p = (unsigned char *)strchr((const char *)str, ';')) != NULL)
     {
@@ -65,7 +65,8 @@ __C_LINK int __UDMCALL udm_wc_mb_8bit(UDM_CONV *conv, UDM_CHARSET *cs, int *wc,
     if(idx->from <= *wc && idx->to >= *wc)
     {
       s[0]=idx->tab[*wc - idx->from];
-      if ((conv->flags & UDM_RECODE_HTML) && (s[0] == '"' || s[0] == '&' || s[0] == '<' || s[0] == '>')) 
+      if ((conv->flags & UDM_RECODE_HTML_SPECIAL) &&
+          (s[0] == '"' || s[0] == '&' || s[0] == '<' || s[0] == '>')) 
         return UDM_CHARSET_ILUNI;
       return (!s[0] && *wc) ? UDM_CHARSET_ILUNI : 1;
     }
