@@ -81,7 +81,7 @@ udm_mb_wc_gujarati(UDM_CONV *conv, UDM_CHARSET *cs, int *pwc,
   conv->icodes = conv->ocodes = 1;
   
   if(hi < 0x80) {
-        if (*s == '&' && (conv->flags & UDM_RECODE_HTML) ) {
+        if (*s == '&' && (conv->flags & UDM_RECODE_HTML_SPECIAL) ) {
 	  if ((p = (unsigned char *)strchr((const char *)s, ';')) != NULL) {
 	    if (s[1] == '#') {
 	      if (s[2] == 'x' || s[2] == 'X') sscanf((const char *)s + 3, "%x;", (unsigned int *)pwc);
@@ -144,7 +144,8 @@ int udm_wc_mb_gujarati(UDM_CONV *conv, UDM_CHARSET *cs, int *wc, unsigned char *
 
   if (*wc < 0x7F) {
     s[0] = *wc;
-    if ((conv->flags && UDM_RECODE_HTML) && (s[0] == '"' || s[0] == '&' || s[0] == '<' || s[0] == '>')) 
+    if ((conv->flags & UDM_RECODE_HTML_SPECIAL) &&
+        (s[0] == '"' || s[0] == '&' || s[0] == '<' || s[0] == '>')) 
       return UDM_CHARSET_ILUNI;
     return conv->ocodes;
   }
