@@ -36,7 +36,7 @@ udm_mb_wc_utf8 (UDM_CONV *conv, UDM_CHARSET *cs,int *pwc,
   conv->icodes = conv->ocodes = 1;
 
   if (c < 0x80) {
-        if (*s == '&' && (conv->flags & UDM_RECODE_HTML) ) {
+        if (*s == '&' && (conv->flags & UDM_RECODE_HTML_SPECIAL) ) {
        if ((p = (unsigned char *)strchr((const char *)s, ';')) != NULL) {
          if (s[1] == '#') {
           if (s[2] == 'x' || s[2] == 'X') sscanf((const char *)s + 3, "%x;", (unsigned int *)pwc);
@@ -107,7 +107,8 @@ udm_wc_mb_utf8(UDM_CONV *conv, UDM_CHARSET *cs, int *wc, unsigned char *r, unsig
 
   if (*wc < 0x80) {
     r[0] = *wc;
-    if ((conv->flags && UDM_RECODE_HTML) && (r[0] == '"' || r[0] == '&' || r[0] == '<' || r[0] == '>')) 
+    if ((conv->flags & UDM_RECODE_HTML_SPECIAL) &&
+        (r[0] == '"' || r[0] == '&' || r[0] == '<' || r[0] == '>')) 
       return UDM_CHARSET_ILUNI;
     return 1;
   }
