@@ -30,7 +30,7 @@
 void *UdmXmalloc(size_t size){
 	void *value;
 
-	value = malloc (size);
+	value = UdmMalloc (size);
 	bzero(value, size);
 	return value;
 }
@@ -41,6 +41,6 @@ void *UdmXrealloc(void *ptr, size_t newsize){
 	if(!ptr)
     	    value = UdmXmalloc(newsize);
 	else
-    	    value = realloc(ptr, newsize);
+    	    value = UdmRealloc(ptr, newsize);
 	return value;
 }
