@@ -60,7 +60,7 @@
 #endif
 
 long tz_offset=0;	/* EXT */
-
+char udm_null_char= 0;
 
 char *UdmStrStore (char *dest, const char *src) {
 	size_t d_size = (dest ? strlen(dest) : 0);
@@ -566,7 +566,7 @@ static int ch2x(int ch)
   return -1;
 }
 
-char * UdmUnescapeCGIQuery(char *d,char *s)
+char * UdmUnescapeCGIQuery(char *d, const char *s)
 {
   char *dd;
   
@@ -920,13 +920,52 @@ time_t UdmHttpDate2Time_t(const char *date){
     if (*date == '\0')
 	return BAD_DATE;
 
+    if (ap_checkmask(date, "####-##-##"))
+    {
+      /*
+         2004-01-02 non-standard format,
+         but we can meet it quit often in user meta tags.
+      */
+      ds.tm_year= ((date[0] - '0') * 10 + (date[1] - '0') - 19) * 100 +
+                  (date[2] - '0') * 10 + date[3] - '0';
+      ds.tm_mon= (date[5] - '0') * 10 + (date[6] - '0');
+      ds.tm_mday= (date[8] - '0') * 10 + (date[9] - '0');
+      ds.tm_hour= 0;
+      ds.tm_min= 0;
+      ds.tm_sec= 0;
+      goto ret; 
+    }
+    else if (ap_checkmask(date, "##.##.####"))
+    {
+      /*
+         0123456789
+         02.01.2005 non-standard format,
+         but we can meet it quit often in
+         user meta tags in Germany.
+      */
+      ds.tm_year= ((date[6] - '0') * 10 + (date[7] - '0') - 19) * 100 +
+                  (date[8] - '0') * 10 + date[9] - '0';
+      ds.tm_mon= (date[3] - '0') * 10 + (date[4] - '0');
+      ds.tm_mday= (date[0] - '0') * 10 + (date[1] - '0');
+      ds.tm_hour= 0;
+      ds.tm_min= 0;
+      ds.tm_sec= 0;
+      goto ret; 
+    }
+
     if ((date = strchr(date, ' ')) == NULL)	/* Find space after weekday */
 	return BAD_DATE;
 
-    ++date;			/* Now pointing to first char after space, which should be */
-    /* start of the actual date information for all 3 formats. */
-
-    if (ap_checkmask(date, "## @$$ #### ##:##:## *")) {	/* RFC 1123 format */
+    ++date;
+    /*
+      Now date points to first char after space, which should be
+      start of the actual date information for all formats.
+    */
+    if (ap_checkmask(date, "## @$$ #### ##:##:## *")) {
+        /*
+          RFC 1123 format
+          Sun, 06 Nov 1994 08:49:37 GMT
+        */
 	ds.tm_year = ((date[7] - '0') * 10 + (date[8] - '0') - 19) * 100;
 	if (ds.tm_year < 0)
 	    return BAD_DATE;
@@ -937,8 +976,26 @@ time_t UdmHttpDate2Time_t(const char *date){
 
 	monstr = date + 3;
 	timstr = date + 12;
+    }                          /*0123456789ABCDEFGHIJK*/
+    else if (ap_checkmask(date, "# @$$ #### ##:##:## *"))
+    {
+        /*
+          The same as above, but with a one-digit day
+          Sun, 6 Nov 1994 08:49:37 GMT
+        */
+	ds.tm_year = ((date[6] - '0') * 10 + (date[7] - '0') - 19) * 100;
+	if (ds.tm_year < 0)
+	    return BAD_DATE;
+	ds.tm_year += ((date[8] - '0') * 10) + (date[9] - '0');
+	ds.tm_mday = date[0] - '0';
+	monstr = date + 2;
+	timstr = date + 11;
     }
-    else if (ap_checkmask(date, "##-@$$-## ##:##:## *")) {		/* RFC 850 format  */
+    else if (ap_checkmask(date, "##-@$$-## ##:##:## *")) {
+       /*
+         RFC 850 format:
+         Sunday, 06-Nov-94 08:49:37 GMT
+       */
 	ds.tm_year = ((date[7] - '0') * 10) + (date[8] - '0');
 	if (ds.tm_year < 70)
 	    ds.tm_year += 100;
@@ -948,7 +1005,11 @@ time_t UdmHttpDate2Time_t(const char *date){
 	monstr = date + 3;
 	timstr = date + 10;
     }
-    else if (ap_checkmask(date, "@$$ ~# ##:##:## ####*")) {	/* asctime format  */
+    else if (ap_checkmask(date, "@$$ ~# ##:##:## ####*")) {
+        /*
+          ANSI C's asctime() format
+          Sun Nov  6 08:49:37 1994
+        */
 	ds.tm_year = ((date[16] - '0') * 10 + (date[17] - '0') - 19) * 100;
 	if (ds.tm_year < 0)
 	    return BAD_DATE;
@@ -1000,6 +1061,7 @@ time_t UdmHttpDate2Time_t(const char *date){
 
     ds.tm_mon = mon;
 
+ret:
     /* printf("parseHTTPdate: %s is %lu\n", date, ap_tm2sec(&ds)); */
 
     return ap_tm2sec(&ds);
@@ -2173,10 +2235,9 @@ void UdmViolationExit(void *v) {
 
 const char udm_soundex[]= "01230120022455012623010202";
 
-static char udm_soundex_code(UDM_CHARSET *cset,char **ptr)
+static char udm_soundex_code(UDM_CHARSET *cset, const char *ptr)
 {
-  unsigned char ch;
-  ch= toupper(**ptr);
+  unsigned char ch= toupper((unsigned char)*ptr);
   if (ch < 'A' || ch > 'Z')
   {
     if (isalpha(ch))
@@ -2193,11 +2254,11 @@ void UdmSoundex(UDM_CHARSET *cs, char *dst, const char *src, size_t srclen)
   const char *srcend= src + srclen;
 
   *dst++= toupper((unsigned char)*src);
-  prev= udm_soundex_code(cs,&src);
+  prev= udm_soundex_code(cs, src);
   end= dst + 3;
 
   src++;
-  while (dst < end && (ch= udm_soundex_code(cs,&src)) != 0 && src < srcend)
+  while (dst < end && (ch= udm_soundex_code(cs, src)) != 0 && src < srcend)
   {
     src++;
     if ((ch != '0') && (ch != prev)) /* if not skipped or double */
@@ -2210,3 +2271,204 @@ void UdmSoundex(UDM_CHARSET *cs, char *dst, const char *src, size_t srclen)
     *dst++ = '0';
   *dst=0;
 }
+
+
+/*
+ * Note that buf must be at least X * 5 + 5 bytes. Where X is
+ * number of integers in array.
+ */
+void udm_zint4_init(UDM_ZINT4_STATE *state, char *buf)
+{
+  state->bits_left= 8;
+  state->prev= 0;
+  state->end= state->buf= (unsigned char *)buf;
+}
+
+
+static inline void udm_zint4_write_bits(UDM_ZINT4_STATE *state,
+    unsigned char byte, unsigned char nbits)
+{
+  if (state->bits_left == 8)
+    *state->end= 0;
+  if (state->bits_left >= nbits)
+  {
+    *state->end+= byte << (state->bits_left - nbits);
+    if (nbits == state->bits_left)
+    {
+      state->end++;
+      state->bits_left= 8;
+    }
+    else
+      state->bits_left-= nbits;
+    return;
+  }
+  *state->end+= byte >> (nbits - state->bits_left);
+  state->end++;
+  state->bits_left+= 8 - nbits;
+  *state->end= byte << state->bits_left;
+}
+
+
+void udm_zint4(UDM_ZINT4_STATE *state, int next)
+{
+  unsigned int n= (unsigned int)(next - state->prev);
+  state->prev= next;
+  if (n < 0x10) /* compressed to 5 bits, 1 block */
+  {
+    udm_zint4_write_bits(state, n, 5);
+  }
+  else if (n < 0x110) /* compressed to 10 bits, 2 blocks */
+  {
+    n-= 0x10;
+    udm_zint4_write_bits(state, 2, 2);
+    udm_zint4_write_bits(state, n, 8);
+  }
+  else if (n < 0x1110) /* compressed to 15 bits, 3 blocks */
+  {
+    n-= 0x110;
+    udm_zint4_write_bits(state, 6, 3);
+    udm_zint4_write_bits(state, n >> 8, 4);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+  }
+  else if (n < 0x11110) /* compressed to 20 bits, 4 blocks */
+  {
+    n-= 0x1110;
+    udm_zint4_write_bits(state, 0xe, 4);
+    udm_zint4_write_bits(state, n >> 8, 8);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+  }
+  else if (n < 0x111110) /* compressed to 25 bits, 5 blocks */
+  {
+    n-= 0x11110;
+    udm_zint4_write_bits(state, 0x1e, 5);
+    udm_zint4_write_bits(state, n >> 16, 4);
+    udm_zint4_write_bits(state, (n >> 8) & 0xff, 8);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+  }
+  else if (n < 0x1111110) /* compressed to 30 bits, 6 blocks */
+  {
+    n-= 0x111110;
+    udm_zint4_write_bits(state, 0x3e, 6);
+    udm_zint4_write_bits(state, n >> 16, 8);
+    udm_zint4_write_bits(state, (n >> 8) & 0xff, 8);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+  }
+  else if (n < 0x11111110) /* compressed to 35 bits */
+  {
+    n-= 0x1111110;
+    udm_zint4_write_bits(state, 0x7e, 7);
+    udm_zint4_write_bits(state, n >> 24, 4);
+    udm_zint4_write_bits(state, (n >> 16) & 0xff, 8);
+    udm_zint4_write_bits(state, (n >> 8) & 0xff, 8);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+  }
+  else /* compressed to 40 bits */
+  {
+    n-= 0x11111110;
+    udm_zint4_write_bits(state, 0xfe, 8);
+    udm_zint4_write_bits(state, n >> 24, 8);
+    udm_zint4_write_bits(state, (n >> 16) & 0xff, 8);
+    udm_zint4_write_bits(state, (n >> 8) & 0xff, 8);
+    udm_zint4_write_bits(state, n & 0xff, 8);
+ }
+}
+
+
+void udm_zint4_finalize(UDM_ZINT4_STATE *state)
+{
+  if (state->bits_left < 8)
+    udm_zint4_write_bits(state, 0xff, 8);
+  *state->end++= 0xff;
+  *state->end++= 0xff;
+  *state->end++= 0xff;
+  *state->end++= 0xff;
+  *state->end++= 0xff;
+}
+
+
+int udm_dezint4(const char *buf, int4 *array, int buf_len)
+{
+  unsigned char next_byte= *buf;
+  unsigned char bit_idx= 8;
+  char nblocks= 1;
+  int4 prev= 0, *array_end;
+  if ((unsigned char)buf[buf_len - 1] != 0xff ||
+      (unsigned char)buf[buf_len - 2] != 0xff ||
+      (unsigned char)buf[buf_len - 3] != 0xff ||
+      (unsigned char)buf[buf_len - 4] != 0xff ||
+      (unsigned char)buf[buf_len - 5] != 0xff)
+    return(0);
+  for (array_end= array;; array_end++)
+  {
+    uint4 next= 0;
+    /* Extract number of blocks */
+    while (next_byte & (1 << --bit_idx))
+    {
+      if (++nblocks == 9)
+        return(array_end - array);
+      if (! bit_idx)
+      {
+        bit_idx= 8;
+        next_byte= *++buf;
+      }
+    }
+    if (! bit_idx)
+    {
+      bit_idx= 8;
+      next_byte= *++buf;
+    }
+
+    for (;; nblocks--)
+    {
+      switch (bit_idx) {
+        case 1:
+          next+= (next_byte & 1) << 3;
+          bit_idx= 5;
+          next_byte= *++buf;
+          next+= (next_byte >> 5) & 0xf;
+          break;
+        case 2:
+          next+= (next_byte & 3) << 2;
+          bit_idx= 6;
+          next_byte= *++buf;
+          next+= (next_byte >> 6) & 7;
+          break;
+        case 3:
+          next+= (next_byte & 7) << 1;
+          bit_idx= 7;
+          next_byte= *++buf;
+          next+= (next_byte >> 7) & 3;
+          break;
+        case 4:
+          next+= next_byte & 0xf;
+          bit_idx= 8;
+          next_byte= *++buf;
+          break;
+        case 5:
+          bit_idx= 1;
+          next+= (next_byte >> 1) & 0xf;
+          break;
+        case 6:
+          bit_idx= 2;
+          next+= (next_byte >> 2) & 0xf;
+          break;
+        case 7:
+          bit_idx= 3;
+          next+= (next_byte >> 3) & 0xf;
+          break;
+        case 8:
+          bit_idx= 4;
+          next+= (next_byte >> 4) & 0xf;
+          break;
+      }
+      if (nblocks > 1)
+        next= (next + 1) << 4;
+      else
+        break;
+    }
+    prev+= next;
+    *array_end= prev;
+  }
+  return(array_end - array);
+}
+
